"""Schema definitions for organization-related data queries."""

from typing import List, Optional

import strawberry
import strawberry_django
import structlog
from django.db.models import Q
from strawberry.types import Info

from api.models import Dataset, Organization, Sector, UseCase
from api.types.type_dataset import TypeDataset
from api.types.type_organization import TypeOrganization
from api.types.type_sector import TypeSector
from api.types.type_usecase import TypeUseCase
from api.utils.enums import DatasetStatus, UseCaseStatus
from api.utils.graphql_telemetry import trace_resolver
from authorization.models import OrganizationMembership, User
from authorization.permissions import IsAuthenticated, IsOrganizationMemberGraphQL
from authorization.types import TypeUser

logger = structlog.get_logger(__name__)


@strawberry.type
class Query:
    """Queries for organization-related data."""

    @strawberry_django.field
    @trace_resolver(
        name="get_organization_published_datasets",
        attributes={"component": "organization"},
    )
    def organization_published_datasets(
        self, info: Info, organization_id: strawberry.ID
    ) -> List[TypeDataset]:
        """Get published datasets for an organization.

        Returns a list of datasets that have been published by the specified organization.
        """
        try:
            # Get published datasets for this organization
            queryset = Dataset.objects.filter(
                organization_id=organization_id, status=DatasetStatus.PUBLISHED.value
            )
            return TypeDataset.from_django_list(queryset)
        except Dataset.DoesNotExist:
            logger.warning(f"No datasets found for organization {organization_id}")
            return []
        except Exception as e:
            logger.error(
                f"Error fetching datasets for organization {organization_id}: {str(e)}"
            )
            return []

    @strawberry_django.field
    @trace_resolver(
        name="get_organization_published_use_cases",
        attributes={"component": "organization"},
    )
    def organization_published_use_cases(
        self, info: Info, organization_id: strawberry.ID
    ) -> List[TypeUseCase]:
        """Get published use cases for an organization.

        Returns a list of use cases that have been published by the specified organization.
        """
        try:
            # Get published use cases for this organization
            queryset = UseCase.objects.filter(
                (
                    Q(organization__id=organization_id)
                    | Q(
                        usecaseorganizationrelationship__organization_id=organization_id
                    )
                ),
                status=UseCaseStatus.PUBLISHED.value,
            ).distinct()
            return TypeUseCase.from_django_list(queryset)
        except UseCase.DoesNotExist:
            logger.warning(f"No use cases found for organization {organization_id}")
            return []
        except Exception as e:
            logger.error(
                f"Error fetching use cases for organization {organization_id}: {str(e)}"
            )
            return []

    @strawberry_django.field
    @trace_resolver(
        name="get_organization_contributed_sectors",
        attributes={"component": "organization"},
    )
    def organization_contributed_sectors(
        self, info: Info, organization_id: strawberry.ID
    ) -> List[TypeSector]:
        """Get sectors that an organization has contributed to.

        Returns a list of unique sectors from all datasets and use cases published by the specified organization.
        """
        try:
            # Get sectors from published datasets
            dataset_sectors = Sector.objects.filter(
                datasets__organization_id=organization_id,
                datasets__status=DatasetStatus.PUBLISHED.value,
            ).distinct()

            # Get sectors from published use cases
            usecase_sectors = Sector.objects.filter(
                usecases__usecaseorganizationrelationship__organization_id=organization_id,
                usecases__status=UseCaseStatus.PUBLISHED.value,
            ).distinct()

            # Combine and deduplicate sectors
            sector_ids = set(dataset_sectors.values_list("id", flat=True))
            sector_ids.update(usecase_sectors.values_list("id", flat=True))

            if not sector_ids:
                return []

            # Get all sectors by their IDs
            queryset = Sector.objects.filter(id__in=sector_ids)
            return TypeSector.from_django_list(queryset)
        except Sector.DoesNotExist:
            logger.warning(f"No sectors found for organization {organization_id}")
            return []
        except Exception as e:
            logger.error(
                f"Error fetching sectors for organization {organization_id}: {str(e)}"
            )
            return []

    @strawberry_django.field
    @trace_resolver(
        name="get_organization_members", attributes={"component": "organization"}
    )
    def organization_members(
        self, info: Info, organization_id: strawberry.ID
    ) -> List[TypeUser]:
        """Get members of an organization.

        Returns a list of users who are members of the specified organization.
        """
        try:
            user_ids = OrganizationMembership.objects.filter(
                organization_id=organization_id
            ).values_list("user_id", flat=True)
            users = User.objects.filter(id__in=user_ids)
            return TypeUser.from_django_list(users)
        except OrganizationMembership.DoesNotExist:
            logger.warning(f"No members found for organization {organization_id}")
            return []
        except Exception as e:
            logger.error(
                f"Error fetching members for organization {organization_id}: {str(e)}"
            )
            return []



import json
import uuid
from typing import Union

from asgiref.sync import sync_to_async
from django.http import HttpRequest, HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt

from api.models import Resource, ResourceChartDetails, ResourceSchema
from api.types.type_resource_chart import chart_base
from api.utils.enums import ChartTypes
from api.views.download_view import generate_chart


async def create_chart_details(
    request_details: dict, resource: Resource
) -> Union[ResourceChartDetails, JsonResponse]:
    """
    Create chart details from request
    """
    options = {}

    # Set chart type
    chart_type = request_details.get("chart_type")

    # Validate chart type
    if chart_type not in ChartTypes.values:
        return JsonResponse(
            {"error": f"Unsupported chart type: {chart_type}"}, status=400
        )

    # Set basic options
    options["x_axis_label"] = request_details.get("x_axis_label", "X-Axis")
    options["y_axis_label"] = request_details.get("y_axis_label", "Y-Axis")
    options["show_legend"] = request_details.get("show_legend", False)
    options["aggregate_type"] = request_details.get("aggregate_type", "none")

    # Handle x-axis column
    if x_axis_column := request_details.get("x_axis_column"):
        options["x_axis_column"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=x_axis_column, resource=resource
        )

    # Handle y-axis columns with configurations
    if y_axis_configs := request_details.get("y_axis_column", []):
        y_axis_columns = []
        for config in y_axis_configs:
            field = await sync_to_async(ResourceSchema.objects.get)(
                field_name=config["field_name"], resource=resource
            )

            # Convert value mapping from list of mappings
            value_mapping = {}
            raw_mappings = config.get("value_mapping", [])
            if len(raw_mappings):
                value_mapping = {
                    str(mapping["key"]): str(mapping["value"])
                    for mapping in raw_mappings
                    if mapping.get("key") is not None
                    and mapping.get("value") is not None
                }

            y_axis_columns.append(
                {
                    "field": field,
                    "label": config.get("label", field.field_name),
                    "color": config.get("color"),
                    "value_mapping": value_mapping,
                    "aggregate_type": config.get("aggregate_type", "none"),
                }
            )

    if y_axis_columns:
        options["y_axis_column"] = y_axis_columns

    # Handle region column
    if region_column := request_details.get("region_column"):
        options["region_column"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=region_column, resource=resource
        )

    # Handle value column
    if value_column := request_details.get("value_column"):
        options["value_column"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=value_column, resource=resource
        )

    # Handle time column for timeline
    if time_column := request_details.get("time_column"):
        options["time_column"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=time_column, resource=resource
        )

    # Enhanced Map Chart specific options
    # Common map options
    if base_map := request_details.get("base_map"):
        options["base_map"] = base_map
    if series_name := request_details.get("series_name"):
        options["series_name"] = series_name
    if name_field := request_details.get("name_field"):
        options["name_field"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=name_field, resource=resource
        )
    if width := request_details.get("width"):
        options["width"] = width
    if height := request_details.get("height"):
        options["height"] = height
    if roam := request_details.get("roam"):
        options["roam"] = roam
    if zoom := request_details.get("zoom"):
        options["zoom"] = zoom
    if center := request_details.get("center"):
        options["center"] = center
    if show_toolbox := request_details.get("show_toolbox"):
        options["show_toolbox"] = show_toolbox

    # Polygon Map Chart options
    if polygon_field := request_details.get("polygon_field"):
        options["polygon_field"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=polygon_field, resource=resource
        )

    # Point Map Chart options
    if lat_field := request_details.get("lat_field"):
        options["lat_field"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=lat_field, resource=resource
        )
    if lng_field := request_details.get("lng_field"):
        options["lng_field"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=lng_field, resource=resource
        )
    if point_size := request_details.get("point_size"):
        options["point_size"] = point_size

    # Geospatial Map Chart options
    if geospatial_field := request_details.get("geospatial_field"):
        options["geospatial_field"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=geospatial_field, resource=resource
        )

    # Extract filters
    request_filters = request_details.get("filters", [])
    filters = []
    for request_filter in request_filters:
        filter = {}
        filter["column"] = await sync_to_async(ResourceSchema.objects.get)(
            field_name=request_filter["column"], resource=resource
        )
        filter["operator"] = request_filter["operator"]
        filter["value"] = request_filter["value"]
        filters.append(filter)

    # Create ResourceChartDetails instance without saving it
    return ResourceChartDetails(
        resource=resource, chart_type=chart_type, options=options, filters=filters
    )


@csrf_exempt
async def generate_dynamic_chart(
    request: HttpRequest, resource_id: uuid.UUID
) -> HttpResponse:
    if request.method == "POST":
        try:
            # Fetch the resource asynchronously
            resource = await sync_to_async(Resource.objects.get)(id=resource_id)
        except Resource.DoesNotExist:
            return JsonResponse({"error": "Resource not found"}, status=404)

        # Validate and process chart details
        chart_details = await create_chart_details(json.loads(request.body), resource)
        if isinstance(chart_details, JsonResponse):
            return chart_details

        # Determine response type (default: json)
        response_type = request.GET.get("response_type", "json").lower()

        # Generate the chart using chart_base
        try:
            chart = await sync_to_async(chart_base)(chart_details)
            if not chart:
                return JsonResponse({"error": "Failed to generate chart"}, status=400)

            if response_type == "file":
                response = await generate_chart(chart_details)
                response["Content-Disposition"] = 'attachment; filename="chart.png"'
                return response

            # Default response: JSON
            return JsonResponse(
                json.loads(chart.dump_options_with_quotes()), safe=False
            )

        except Exception as e:
            return JsonResponse({"error": f"Error generating chart: {e}"}, status=500)

    return JsonResponse({"error": "Invalid HTTP method"}, status=405)



"""Register all activity tracking handlers."""

# mypy: disable-error-code=arg-type
# mypy: disable-error-code=union-attr

from typing import Any, Dict, Optional

from django.http import HttpRequest

# Import all specialized tracking functions
from api.activities.dataset import (
    track_dataset_created,
    track_dataset_published,
    track_dataset_updated,
)
from api.activities.organization import (
    track_organization_created,
    track_organization_updated,
)
from api.activities.registry import register_model_handler
from api.activities.resource import (
    track_resource_created,
    track_resource_downloaded,
    track_resource_previewed,
    track_resource_updated,
)
from api.activities.usecase import (
    track_usecase_created,
    track_usecase_published,
    track_usecase_updated,
)
from authorization.models import User

# Register Dataset handlers
# Register Dataset handlers
register_model_handler(
    "Dataset",
    "created",
    lambda user, instance, data, request: track_dataset_created(
        user, instance, request
    ),
)

register_model_handler(
    "Dataset",
    "updated",
    lambda user, instance, data, request: track_dataset_updated(
        user, instance, data, request
    ),
)

register_model_handler(
    "Dataset",
    "published",
    lambda user, instance, data, request: track_dataset_published(
        user, instance, request
    ),
)

register_model_handler(
    "Dataset",
    "unpublished",
    lambda user, instance, data, request: track_dataset_updated(
        user, instance, {"status": "DRAFT", "action": "unpublished"}, request
    ),
)

# Register Organization handlers
register_model_handler(
    "Organization",
    "created",
    lambda user, instance, data, request: track_organization_created(
        user, instance, request
    ),
)

register_model_handler(
    "Organization",
    "updated",
    lambda user, instance, data, request: track_organization_updated(
        user, instance, data, request
    ),
)

register_model_handler(
    "Organization",
    "deleted",
    lambda user, instance, data, request: track_organization_updated(
        user,
        instance,
        {"action": "deleted", "organization_id": data.get("organization_id")},
        request,
    ),
)

# Register Resource handlers
register_model_handler(
    "Resource",
    "created",
    lambda user, instance, data, request: track_resource_created(
        user, instance, request
    ),
)

register_model_handler(
    "Resource",
    "updated",
    lambda user, instance, data, request: track_resource_updated(
        user, instance, data, request
    ),
)

register_model_handler(
    "Resource",
    "downloaded",
    lambda user, instance, data, request: track_resource_downloaded(
        user, instance, request
    ),
)

register_model_handler(
    "Resource",
    "previewed",
    lambda user, instance, data, request: track_resource_previewed(
        user, instance, request
    ),
)

register_model_handler(
    "Resource",
    "versioned",
    lambda user, instance, data, request: track_resource_updated(
        user,
        instance,
        {
            "action": "versioned",
            "version": data.get("version"),
            "description": data.get("description"),
        },
        request,
    ),
)

register_model_handler(
    "Resource",
    "deleted",
    lambda user, instance, data, request: track_resource_updated(
        user,
        instance,
        {"action": "deleted", "resource_id": data.get("resource_id")},
        request,
    ),
)

# Register UseCase handlers
register_model_handler(
    "UseCase",
    "created",
    lambda user, instance, data, request: track_usecase_created(
        user, instance, request
    ),
)

register_model_handler(
    "UseCase",
    "updated",
    lambda user, instance, data, request: track_usecase_updated(
        user, instance, data, request
    ),
)

register_model_handler(
    "UseCase",
    "published",
    lambda user, instance, data, request: track_usecase_published(
        user, instance, request
    ),
)

register_model_handler(
    "UseCase",
    "unpublished",
    lambda user, instance, data, request: track_usecase_updated(
        user, instance, {"status": "DRAFT", "action": "unpublished"}, request
    ),
)

register_model_handler(
    "UseCase",
    "deleted",
    lambda user, instance, data, request: track_usecase_updated(
        user,
        instance,
        {"action": "deleted", "usecase_id": data.get("usecase_id")},
        request,
    ),
)



import unittest
from unittest.mock import MagicMock

import pandas as pd

from api.types.charts.unified_chart import UnifiedChart


class MockResourceChartDetails:
    """Mock class for ResourceChartDetails to avoid Django dependency"""

    def __init__(
        self,
        chart_type="BAR",
        title="Test Chart",
        description="Test Description",
        options=None,
    ):
        self.chart_type = chart_type
        self.title = title
        self.description = description
        self.options = options or {}
        self.filters = []


class TestGroupedBarChart(unittest.TestCase):
    def setUp(self):
        """Set up test fixtures."""
        # Sample data for testing
        self.test_data = pd.DataFrame(
            {
                "category": ["A", "A", "B", "B", "C", "C"],
                "group": ["G1", "G2", "G1", "G2", "G1", "G2"],
                "value": [10, 15, 20, 25, 30, 35],
                "time": [
                    "2023-01",
                    "2023-01",
                    "2023-02",
                    "2023-02",
                    "2023-03",
                    "2023-03",
                ],
            }
        )

        # Mock field objects
        self.x_field = MagicMock()
        self.x_field.field_name = "category"

        self.y_field = MagicMock()
        self.y_field.field_name = "value"

        self.group_field = MagicMock()
        self.group_field.field_name = "group"

        self.time_field = MagicMock()
        self.time_field.field_name = "time"

    def test_vertical_grouped_bar_chart(self):
        """Test vertical grouped bar chart creation."""
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": {"field": self.y_field, "label": "Value"},
            "group_column": self.group_field,
        }
        chart_details = MockResourceChartDetails(
            chart_type="BAR",
            options={**options, "allow_multi_series": true},
            options=options,
        )
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        self.assertIsNotNone(result)
        self.assertEqual(len(result.options.get("series")), 2)  # Two groups
        self.assertEqual(result.options.get("xAxis")[0]["type"], "category")
        self.assertEqual(result.options.get("yAxis")[0]["type"], "value")

    def test_horizontal_grouped_bar_chart(self):
        """Test horizontal grouped bar chart creation."""
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": {"field": self.y_field, "label": "Value"},
            "group_column": self.group_field,
        }
        chart_details = MockResourceChartDetails(
            chart_type="GROUPED_BAR_HORIZONTAL", options=options
        )
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        self.assertIsNotNone(result)
        self.assertEqual(len(result.options.get("series")), 2)  # Two groups
        self.assertEqual(result.options.get("xAxis")[0]["type"], "value")
        self.assertEqual(result.options.get("yAxis")[0]["type"], "category")

    def test_styling_options(self):
        """Test styling options for grouped bar chart."""
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": {"field": self.y_field, "label": "Value"},
            "group_column": self.group_field,
            "colors": ["#FF0000", "#00FF00"],
        }
        chart_details = MockResourceChartDetails(
            chart_type="BAR",
            options={**options, "allow_multi_series": true},
            options=options,
        )
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        series = result.options.get("series")
        self.assertEqual(series[0].get("itemStyle").get("color"), "#FF0000")
        self.assertEqual(series[1].get("itemStyle").get("color"), "#00FF00")

    def test_value_aggregation(self):
        """Test value aggregation in grouped bar chart."""
        # Create test data with duplicate categories and groups
        test_data = pd.DataFrame(
            {
                "category": ["A", "A", "B", "B"],
                "group": ["G1", "G1", "G2", "G2"],
                "value": [10, 20, 30, 40],
            }
        )

        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": {
                "field": self.y_field,
                "label": "Value",
                "aggregate_type": "AVERAGE",
            },
            "group_column": self.group_field,
        }
        chart_details = MockResourceChartDetails(
            chart_type="BAR",
            options={**options, "allow_multi_series": true},
            options=options,
        )
        chart = UnifiedChart(chart_details, test_data)
        result = chart.create_chart()

        series = result.options.get("series")
        # Check average values
        self.assertEqual(series[0].get("data")[0], 15.0)  # (10 + 20) / 2
        self.assertEqual(series[1].get("data")[0], 35.0)  # (30 + 40) / 2

    def test_time_based_grouped_bar_chart(self):
        """Test time-based grouped bar chart."""
        options = {
            "x_axis_column": self.time_field,
            "y_axis_column": {"field": self.y_field, "label": "Value"},
            "group_column": self.group_field,
        }
        chart_details = MockResourceChartDetails(
            chart_type="BAR",
            options={**options, "allow_multi_series": true},
            options=options,
        )
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        x_axis = result.options.get("xAxis")[0]
        self.assertEqual(len(x_axis.get("data")), 3)  # Three unique time periods
        self.assertTrue(
            all(
                period in x_axis.get("data")
                for period in ["2023-01", "2023-02", "2023-03"]
            )
        )

    def test_value_mapping(self):
        """Test value mapping in grouped bar chart."""
        # Create test data with string values that need mapping
        test_data = pd.DataFrame(
            {"category": ["A", "B"], "group": ["G1", "G2"], "value": ["High", "Low"]}
        )

        value_map = {"High": 100, "Low": 50}
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": {
                "field": self.y_field,
                "label": "Value",
                "value_map": value_map,
            },
            "group_column": self.group_field,
        }
        chart_details = MockResourceChartDetails(
            chart_type="BAR",
            options={**options, "allow_multi_series": true},
            options=options,
        )
        chart = UnifiedChart(chart_details, test_data)
        result = chart.create_chart()

        series = result.options.get("series")
        self.assertEqual(series[0].get("data")[0], 100)  # 'High' mapped to 100
        self.assertEqual(series[1].get("data")[0], 50)  # 'Low' mapped to 50


if __name__ == "__main__":
    unittest.main()



from typing import Any

from django.db import models
from django.utils.text import slugify

from api.utils.file_paths import _dataspace_directory_path


class DataSpace(models.Model):
    name = models.CharField(max_length=200)
    description = models.CharField(max_length=1000)
    logo = models.ImageField(upload_to=_dataspace_directory_path, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    homepage = models.URLField(blank=True)
    contact_email = models.EmailField(blank=True, null=True)
    slug = models.SlugField(max_length=75, null=True, blank=False, unique=True)

    def save(self, *args: Any, **kwargs: Any) -> None:
        self.slug = slugify(self.name)
        super().save(*args, **kwargs)

    class Meta:
        db_table = "dataspace"



from typing import Optional

import strawberry
import strawberry_django
from strawberry import auto

from api.models import SDG
from api.types.base_type import BaseType


@strawberry_django.filter(SDG)
class SDGFilter:
    """Filter class for SDG model."""

    id: auto
    code: auto
    name: auto


@strawberry_django.order(SDG)
class SDGOrder:
    """Order class for SDG model."""

    number: auto
    code: auto
    name: auto


@strawberry_django.type(
    SDG,
    pagination=True,
    fields="__all__",
    filters=SDGFilter,
    order=SDGOrder,  # type: ignore
)
class TypeSDG(BaseType):
    """GraphQL type for SDG model."""

    pass



"""GraphQL types for AI Model."""

import uuid
from datetime import datetime
from typing import TYPE_CHECKING, Any, List, Optional, cast

import strawberry
import strawberry_django
import structlog
from strawberry.enum import EnumType
from strawberry.types import Info

from api.models.AIModel import AIModel, ModelEndpoint
from api.types.base_type import BaseType
from api.types.type_dataset import TypeTag
from api.types.type_geo import TypeGeo
from api.types.type_organization import TypeOrganization
from api.types.type_sector import TypeSector
from api.utils.enums import (
    AIModelProvider,
    AIModelStatus,
    AIModelType,
    EndpointAuthType,
    EndpointHTTPMethod,
)
from authorization.types import TypeUser

logger = structlog.get_logger("dataspace.type_aimodel")


# Create GraphQL enums from Django TextChoices
AIModelTypeEnum = strawberry.enum(AIModelType)  # type: ignore
AIModelStatusEnum = strawberry.enum(AIModelStatus)  # type: ignore
AIModelProviderEnum = strawberry.enum(AIModelProvider)  # type: ignore
EndpointAuthTypeEnum = strawberry.enum(EndpointAuthType)  # type: ignore
EndpointHTTPMethodEnum = strawberry.enum(EndpointHTTPMethod)  # type: ignore


@strawberry.type
class TypeModelEndpoint(BaseType):
    """GraphQL type for ModelEndpoint."""

    id: int
    url: str
    http_method: EndpointHTTPMethodEnum
    auth_type: EndpointAuthTypeEnum
    auth_header_name: str
    headers: strawberry.scalars.JSON
    request_template: strawberry.scalars.JSON
    response_path: Optional[str]
    timeout_seconds: int
    max_retries: int
    is_primary: bool
    is_active: bool
    rate_limit_per_minute: Optional[int]
    last_success_at: Optional[datetime]
    last_failure_at: Optional[datetime]
    total_requests: int
    failed_requests: int
    created_at: datetime
    updated_at: datetime

    @strawberry.field
    def success_rate(self) -> Optional[float]:
        """Calculate success rate."""
        if self.total_requests == 0:
            return None
        return (
            (self.total_requests - self.failed_requests) / self.total_requests
        ) * 100


@strawberry_django.filter(AIModel)
class AIModelFilter:
    """Filter for AI Model."""

    id: Optional[int]
    status: Optional[AIModelStatusEnum]
    model_type: Optional[AIModelTypeEnum]
    provider: Optional[AIModelProviderEnum]
    is_public: Optional[bool]
    is_active: Optional[bool]


@strawberry_django.order(AIModel)
class AIModelOrder:
    """Order for AI Model."""

    name: strawberry.auto
    display_name: strawberry.auto
    created_at: strawberry.auto
    updated_at: strawberry.auto
    last_tested_at: strawberry.auto


@strawberry_django.type(
    AIModel,
    fields="__all__",
    filters=AIModelFilter,
    pagination=True,
    order=AIModelOrder,  # type: ignore
)
class TypeAIModel(BaseType):
    """GraphQL type for AI Model."""

    id: int
    name: str
    display_name: str
    version: Optional[str]
    description: str
    model_type: AIModelTypeEnum
    provider: AIModelProviderEnum
    provider_model_id: Optional[str]
    organization: Optional["TypeOrganization"]
    user: Optional["TypeUser"]
    supports_streaming: bool
    max_tokens: Optional[int]
    supported_languages: strawberry.scalars.JSON
    input_schema: strawberry.scalars.JSON
    output_schema: strawberry.scalars.JSON
    metadata: strawberry.scalars.JSON
    status: AIModelStatusEnum
    is_public: bool
    is_active: bool
    average_latency_ms: Optional[float]
    success_rate: Optional[float]
    last_audit_score: Optional[float]
    audit_count: int
    created_at: datetime
    updated_at: datetime
    last_tested_at: Optional[datetime]

    @strawberry.field
    def endpoints(self, info: Info) -> List[TypeModelEndpoint]:
        """Get endpoints for this AI model.

        Args:
            info: Request info

        Returns:
            List[TypeModelEndpoint]: List of endpoints
        """
        try:
            django_instance = cast(AIModel, self)
            queryset = django_instance.endpoints.all()
            return TypeModelEndpoint.from_django_list(list(queryset))
        except (AttributeError, AIModel.DoesNotExist):
            return []

    @strawberry.field(description="Get tags associated with this AI model.")
    def tags(self) -> Optional[List[TypeTag]]:
        """Get tags associated with this AI model."""
        try:
            queryset = self.tags.all()  # type: ignore
            if not queryset.exists():
                return []
            return TypeTag.from_django_list(queryset)
        except Exception:
            return []

    @strawberry.field(description="Get sectors associated with this AI model.")
    def sectors(self) -> Optional[List[TypeSector]]:
        """Get sectors associated with this AI model."""
        try:
            queryset = self.sectors.all()  # type: ignore
            if not queryset.exists():
                return []
            return TypeSector.from_django_list(queryset)
        except Exception:
            return []

    @strawberry.field(description="Get geographies associated with this AI model.")
    def geographies(self) -> Optional[List[TypeGeo]]:
        """Get geographies associated with this AI model."""
        try:
            queryset = self.geographies.all()  # type: ignore
            if not queryset.exists():
                return []
            return TypeGeo.from_django_list(queryset)
        except Exception:
            return []

    @strawberry.field(description="Get primary endpoint for this model.")
    def primary_endpoint(self) -> Optional[TypeModelEndpoint]:
        """Get primary endpoint for this model."""
        endpoint = self.get_primary_endpoint()  # type: ignore
        if endpoint:
            return TypeModelEndpoint.from_django(endpoint)
        return None



from typing import List, Tuple

from django.db import models


class OrganizationTypes(models.TextChoices):
    STATE_GOVERNMENT = "STATE GOVERNMENT"
    UNION_TERRITORY_GOVERNMENT = "UNION TERRITORY GOVERNMENT"
    URBAN_LOCAL_BODY = "URBAN LOCAL BODY"
    ACADEMIC_INSTITUTION = "ACADEMIC INSTITUTION"
    CENTRAL_GOVERNMENT = "CENTRAL GOVERNMENT"
    CITIZENS_GROUP = "CITIZENS GROUP"
    CIVIL_SOCIETY_ORGANISATION = "CIVIL SOCIETY ORGANISATION"
    INDUSTRY_BODY = "INDUSTRY BODY"
    MEDIA_ORGANISATION = "MEDIA ORGANISATION"
    OPEN_DATA_TECHNOLOGY_COMMUNITY = "OPEN DATA/TECHNOLOGY COMMUNITY"
    PRIVATE_COMPANY = "PRIVATE COMPANY"
    PUBLIC_SECTOR_COMPANY = "PUBLIC SECTOR COMPANY"
    OTHERS = "OTHERS"
    STARTUP = "STARTUP"
    GOVERNMENT = "GOVERNMENT"
    CORPORATIONS = "CORPORATIONS"
    NGO = "NGO"


class GeoTypes(models.TextChoices):
    REGION = "REGION"
    COUNTRY = "COUNTRY"
    STATE = "STATE"
    DISTRICT = "DISTRICT"
    UT = "UT"


class DataType(models.TextChoices):
    API = "API"
    FILE = "FILE"
    EXTERNAL = "EXTERNAL"


class MetadataModels(models.TextChoices):
    DATASET = "DATASET"
    RESEOURCE = "RESOURCE"
    USECASE = "USECASE"
    COLLABORATIVE = "COLLABORATIVE"


class MetadataStandards(models.TextChoices):
    DCATV3 = "DCATV3"
    OCDS = "OCDS"
    OBDS = "OBDS"
    NA = "NA"


class MetadataDataTypes(models.TextChoices):
    STRING = "STRING"
    NUMBER = "NUMBER"
    SELECT = "SELECT"
    MULTISELECT = "MULTISELECT"
    DATE = "DATE"
    URL = "URL"


class MetadataTypes(models.TextChoices):
    REQUIRED = "REQUIRED"
    OPTIONAL = "OPTIONAL"
    ADVANCED = "ADVANCED"


class AccessTypes(models.TextChoices):
    PUBLIC = "PUBLIC"
    PRIVATE = "PRIVATE"
    PROTECTED = "PROTECTED"


class FieldTypes(models.TextChoices):
    STRING = "STRING"
    NUMBER = "NUMBER"
    INTEGER = "INTEGER"
    DATE = "DATE"
    BOOLEAN = "BOOLEAN"


class DatasetStatus(models.TextChoices):
    DRAFT = "DRAFT"
    PUBLISHED = "PUBLISHED"
    ARCHIVED = "ARCHIVED"


class DatasetAccessType(models.TextChoices):
    PUBLIC = "PUBLIC"
    PRIVATE = "PRIVATE"
    RESTRICTED = "RESTRICTED"


class DatasetLicense(models.TextChoices):
    GOVERNMENT_OPEN_DATA_LICENSE = "GOVERNMENT_OPEN_DATA_LICENSE"
    CC_BY_4_0_ATTRIBUTION = "CC_BY_4_0_ATTRIBUTION"
    CC_BY_SA_4_0_ATTRIBUTION_SHARE_ALIKE = "CC_BY_SA_4_0_ATTRIBUTION_SHARE_ALIKE"
    OPEN_DATA_COMMONS_BY_ATTRIBUTION = "OPEN_DATA_COMMONS_BY_ATTRIBUTION"
    OPEN_DATABASE_LICENSE = "OPEN_DATABASE_LICENSE"


class UseCaseStatus(models.TextChoices):
    DRAFT = "DRAFT"
    PUBLISHED = "PUBLISHED"
    ARCHIVED = "ARCHIVED"


class CollaborativeStatus(models.TextChoices):
    DRAFT = "DRAFT"
    PUBLISHED = "PUBLISHED"
    ARCHIVED = "ARCHIVED"


class UseCaseRunningStatus(models.TextChoices):
    INITIATED = "INITIATED"
    ON_GOING = "ON_GOING"
    COMPLETED = "COMPLETED"
    ON_HOLD = "ON_HOLD"
    CANCELLED = "CANCELLED"


class ChartTypes(models.TextChoices):
    # New simplified chart types
    BAR = "BAR"
    LINE = "LINE"
    BIG_NUMBER = "BIG_NUMBER"

    # Map and other specialized charts
    ASSAM_DISTRICT = "ASSAM_DISTRICT"
    ASSAM_RC = "ASSAM_RC"
    TREEMAP = "TREEMAP"

    # Enhanced map chart types
    POLYGON_MAP = "POLYGON_MAP"
    POINT_MAP = "POINT_MAP"
    GEOSPATIAL_MAP = "GEOSPATIAL_MAP"


class ChartStatus(models.TextChoices):
    DRAFT = "DRAFT"
    PUBLISHED = "PUBLISHED"
    ARCHIVED = "ARCHIVED"


class AggregateType(models.TextChoices):
    NONE = "NONE"
    SUM = "SUM"
    AVERAGE = "AVG"
    COUNT = "COUNT"


class OrganizationRelationshipType(models.TextChoices):
    SUPPORTER = "SUPPORTER"
    PARTNER = "PARTNER"


class AIModelType(models.TextChoices):
    TRANSLATION = "TRANSLATION"
    TEXT_GENERATION = "TEXT_GENERATION"
    SUMMARIZATION = "SUMMARIZATION"
    QUESTION_ANSWERING = "QUESTION_ANSWERING"
    SENTIMENT_ANALYSIS = "SENTIMENT_ANALYSIS"
    TEXT_CLASSIFICATION = "TEXT_CLASSIFICATION"
    NAMED_ENTITY_RECOGNITION = "NAMED_ENTITY_RECOGNITION"
    TEXT_TO_SPEECH = "TEXT_TO_SPEECH"
    SPEECH_TO_TEXT = "SPEECH_TO_TEXT"
    OTHER = "OTHER"


class AIModelStatus(models.TextChoices):
    REGISTERED = "REGISTERED"
    VALIDATING = "VALIDATING"
    ACTIVE = "ACTIVE"
    AUDITING = "AUDITING"
    APPROVED = "APPROVED"
    FLAGGED = "FLAGGED"
    DEPRECATED = "DEPRECATED"


class AIModelProvider(models.TextChoices):
    OPENAI = "OPENAI"
    LLAMA_OLLAMA = "LLAMA_OLLAMA"
    LLAMA_TOGETHER = "LLAMA_TOGETHER"
    LLAMA_REPLICATE = "LLAMA_REPLICATE"
    LLAMA_CUSTOM = "LLAMA_CUSTOM"
    CUSTOM = "CUSTOM"
    HUGGINGFACE = "HUGGINGFACE"


class AIModelFramework(models.TextChoices):
    PYTORCH = "pt", "PyTorch"
    TENSORFLOW = "tf", "TensorFlow"


class HFModelClass(models.TextChoices):
    CAUSAL_LM = "AutoModelForCausalLM", "Causal Language Modeling Head"
    SEQ2SEQ_LM = "AutoModelForSeq2SeqLM", "Sequence-To-Sequence Language Modeling Head"
    SEQUENCE_CLASSIFICATION = "AutoModelForSequenceClassification", "Sequence Classification Head"
    NEXT_SENTENCE_PREDICTION = "AutoModelForNextSentencePrediction", "Next Sentence Prediction Head"
    MULTIPLE_CHOICE = "AutoModelForMultipleChoice", "Multiple Choice Head"
    TOKEN_CLASSIFICATION = "AutoModelForTokenClassification", "Token Classification Head"
    QUESTION_ANSWERING = "AutoModelForQuestionAnswering", "Question Answering Head"
    MASKED_LM = "AutoModelForMaskedLM", "Masked Language Modeling Head"


class EndpointHTTPMethod(models.TextChoices):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"


class EndpointAuthType(models.TextChoices):
    BEARER = "BEARER"
    API_KEY = "API_KEY"
    BASIC = "BASIC"
    OAUTH2 = "OAUTH2"
    CUSTOM = "CUSTOM"
    NONE = "NONE"



from typing import Any, Dict, List, Optional, Protocol, Union, cast

import numpy as np
import pandas as pd
import structlog
from pyecharts import options as opts
from pyecharts.charts import Bar, Line, Map, TreeMap
from pyecharts.charts.chart import Chart

from api.models import ResourceChartDetails
from api.utils.data_indexing import query_resource_data
from api.utils.enums import AggregateType

logger = structlog.get_logger("dataspace.charts")


class DjangoFieldLike(Protocol):
    """Protocol for Django-like field objects."""

    field_name: str


class ChartFilter(Protocol):
    column: DjangoFieldLike
    operator: str
    value: str


class ChartOptions(Protocol):
    x_axis_column: DjangoFieldLike
    y_axis_column: List[Dict[str, Any]]
    time_column: Optional[DjangoFieldLike]
    time_groups: Optional[List[str]]
    width: str
    height: str


CHART_TYPE_MAP = {
    "LINE": Line,
    "ASSAM_DISTRICT": Map,
    "ASSAM_RC": Map,
    "POLYGON_MAP": Map,
    "POINT_MAP": Map,
    "GEOSPATIAL_MAP": Map,
    "TREEMAP": TreeMap,
    "BAR": Bar,
    "BIG_NUMBER": Bar,
}


class BaseChart:
    """Base class for all chart types."""

    def __init__(self, chart_details: ResourceChartDetails):
        """Initialize chart with details."""
        self.chart_details = chart_details
        self.options: Dict[str, Union[DjangoFieldLike, Dict[str, Any], List, Any]] = (
            chart_details.options
        )
        self.filters = chart_details.filters or []

    def get_chart_class(self) -> Chart:
        """Get the chart class to use."""
        return CHART_TYPE_MAP.get(self.chart_details.chart_type)

    def _get_data(self) -> Optional[pd.DataFrame]:
        """Get data for the chart using SQL query."""
        try:
            query, params = self._build_sql_query()
            return query_resource_data(self.chart_details.resource, query)
        except Exception as e:
            logger.error(f"Error getting chart data: {str(e)}")
            return None

    def create_chart(self) -> Optional[Chart]:
        """Create a chart with the given data and options."""
        try:
            # Get filtered data
            filtered_data = self._get_data()
            if filtered_data is None or filtered_data.empty:
                return None

            # Initialize chart
            chart = self.initialize_chart(filtered_data)

            # Configure chart
            self.configure_chart(chart, filtered_data)

            chart.set_global_opts(
                legend_opts=opts.LegendOpts(
                    is_show=self.options.get("show_legend", True),
                    pos_bottom="0%",  # Position at the very bottom
                    pos_top="auto",
                    orient="horizontal",
                    textstyle_opts=opts.TextStyleOpts(font_size=12),
                ),
                toolbox_opts=opts.ToolboxOpts(
                    is_show=True,
                    orient="vertical",
                    pos_left=None,  # Explicitly set to None to override the default 80%
                    pos_right="0%",
                    pos_top="center",
                    feature={
                        "saveAsImage": {
                            "show": True,
                            "title": "",
                            "type": [
                                "png",
                                "jpeg",
                                "svg",
                            ],  # Support multiple export formats
                            "pixelRatio": 2,  # Higher quality export
                            "backgroundColor": "#fff",
                        },
                        "dataView": {
                            "show": True,
                            "title": "",
                            "readOnly": False,  # Allow editing in data view
                            "lang": [
                                "Data View",
                                "Close",
                                "Refresh",
                            ],  # Customize buttons
                        },
                        "restore": {"show": True, "title": ""},
                        "dataZoom": {"show": True, "title": ""},
                    },
                ),
                datazoom_opts=[
                    opts.DataZoomOpts(
                        is_show=True,
                        type_="slider",
                        range_start=0,
                        range_end=100,
                        pos_bottom="10%",
                        xaxis_index=[0],
                    ),
                    opts.DataZoomOpts(type_="inside"),
                ],
            )

            return chart
        except Exception as e:
            import traceback

            logger.error(
                f"Error creating chart: {str(e)} , traceback: {traceback.format_exc()}"
            )
            return None

    def _get_sql_function_for_aggregate_type(self, agg_type: AggregateType) -> str:
        """Convert AggregateType enum value to the corresponding SQL function name.

        Args:
            agg_type: The aggregation type enum value

        Returns:
            The SQL function name to use in the query
        """
        # Map AggregateType enum values to SQL function names
        if agg_type == AggregateType.SUM:
            return "sum"
        elif agg_type == AggregateType.AVERAGE:
            return "avg"
        elif agg_type == AggregateType.COUNT:
            return "count"
        else:
            # Default to sum if unknown
            logger.warning(f"Unknown aggregation type: {agg_type}, defaulting to sum")
            return "sum"

    def _process_value(self, value: str, operator: str) -> Any:
        """Process the filter value based on the operator."""
        if operator in ["contains", "not_contains"]:
            return f"%{value}%"
        elif operator in ["in", "not_in"]:
            return value.split(",") if "," in value else [value]
        return value

    def _build_sql_filter(self, filter_dict: Dict[str, Any]) -> tuple[str, Any]:
        """Build SQL WHERE clause from filter dict."""
        column = cast(DjangoFieldLike, filter_dict.get("column", {}))
        operator = filter_dict.get("operator", "")
        value = self._process_value(filter_dict.get("value", ""), operator)

        field_name = column.field_name
        if not field_name:
            return "", None

        operator_map = {
            "equals": "=",
            "not_equals": "!=",
            "greater_than": ">",
            "less_than": "<",
            "greater_than_equals": ">=",
            "less_than_equals": "<=",
            "contains": "LIKE",
            "not_contains": "NOT LIKE",
            "in": "IN",
            "not_in": "NOT IN",
        }

        sql_operator = operator_map.get(operator)
        if not sql_operator:
            return "", None

        if operator in ["in", "not_in"]:
            placeholders = ",".join(["%s"] * len(value))
            return f'"{field_name}" {sql_operator} ({placeholders})', value

        return f'"{field_name}" {sql_operator} %s', value

    def _build_sql_query(self) -> tuple[str, List[Any]]:
        """Build SQL query from chart options."""
        # Get columns
        select_cols = []
        params = []

        x_axis_col = cast(DjangoFieldLike, self.options.get("x_axis_column", {}))
        if x_col := x_axis_col.field_name:
            select_cols.append(f'"{x_col}"')

        y_axis_cols = self._get_y_axis_columns()
        for y_col in y_axis_cols:
            if field := y_col.get("field"):
                field_name = field.field_name
                select_cols.append(f'"{field_name}"')

        # Handle time-based data
        time_column = cast(DjangoFieldLike, self.options.get("time_column", {}))
        if time_column:
            time_field = time_column.field_name
            if time_field and time_field not in select_cols:
                select_cols.append(f'"{time_field}"')

        # Build query
        query = f"SELECT {', '.join(select_cols)} FROM {{{{table}}}}"

        # Add filters
        where_clauses = []
        for filter_dict in self.filters:
            clause, value = self._build_sql_filter(filter_dict)
            if clause:
                where_clauses.append(clause)
                if isinstance(value, list):
                    params.extend(value)
                else:
                    params.append(value)

        if where_clauses:
            query += f" WHERE {' AND '.join(where_clauses)}"

        # Handle aggregation
        agg_type = cast(
            AggregateType, self.options.get("aggregate_type", AggregateType.NONE)
        )
        group_by = []

        # Determine if we need to group by any columns
        if agg_type != AggregateType.NONE:
            x_col = x_axis_col.field_name
            if x_col:
                group_by.append(f'"{x_col}"')
            if time_column:
                time_field = time_column.field_name
                if time_field:
                    group_by.append(f'"{time_field}"')

        # Always rebuild the select columns for consistent handling
        # Remove y-axis columns from the select list as we'll add them back with proper handling
        select_cols = [
            col
            for col in select_cols
            if not any(
                y_col.get("field", {}).field_name in col for y_col in y_axis_cols
            )
        ]

        # Add back y-axis columns with proper handling based on aggregation type
        for y_col in y_axis_cols:
            if field := y_col.get("field"):
                field_name = field.field_name
                if agg_type == AggregateType.NONE:
                    # For NONE, just use the column name directly
                    select_cols.append(f'"{field_name}"')
                else:
                    # For other aggregation types, wrap in the appropriate function
                    # Need to convert from enum value to SQL function name
                    sql_function = self._get_sql_function_for_aggregate_type(agg_type)
                    select_cols.append(
                        f'{sql_function}("{field_name}") as "{field_name}"'
                    )

        # Rebuild the query with the updated select columns
        query = f"SELECT {', '.join(select_cols)} FROM {{{{table}}}}"

        # Add group by
        if group_by:
            query += f" GROUP BY {', '.join(group_by)}"

        # Add order by
        order_by = []
        if time_column:
            time_field = time_column.field_name
            if time_field:
                order_by.append(f'"{time_field}"')
        if x_col := x_axis_col.field_name:
            order_by.append(f'"{x_col}"')

        if order_by:
            sort_order_value = self.options.get("sort_order", "asc")
            if isinstance(sort_order_value, str):
                sort_order = sort_order_value.upper()
            else:
                sort_order = "ASC"  # Default to ASC if not a string
            query += f' ORDER BY {", ".join(order_by)} {sort_order}'

        return query, params

    def get_y_axis_bounds(self) -> tuple[float, float]:
        """Calculate min and max bounds for y-axis."""
        try:
            data = self._get_data()
            if data is None or data.empty:
                return 0, 5

            y_values = []
            for y_axis_column in self._get_y_axis_columns():
                if field := y_axis_column.get("field"):
                    field_name = field.field_name

                    # Check if the field exists in the DataFrame
                    if field_name not in data.columns:
                        logger.warning(
                            f"Field '{field_name}' not found in data columns"
                        )
                        continue

                    # Get the column data directly
                    column_data = data[field_name]

                    # Handle case where column_data is a DataFrame (can happen with SQL queries)
                    if isinstance(column_data, pd.DataFrame):
                        logger.debug(
                            f"Column {field_name} is a DataFrame, using first column"
                        )
                        if column_data.empty:
                            continue
                        column_data = column_data.iloc[:, 0]  # Take the first column

                    # Drop NA values and convert to float
                    try:
                        # For Series, convert to float and extend y_values
                        clean_values = column_data.dropna()
                        if not clean_values.empty:
                            float_values = clean_values.astype(float).tolist()
                            y_values.extend(float_values)
                    except (ValueError, TypeError) as e:
                        logger.warning(f"Error converting values for {field_name}: {e}")
                    except Exception as e:
                        logger.warning(
                            f"Unexpected error processing values for {field_name}: {e}"
                        )

            if not y_values:
                return 0, 5

            min_val = min(y_values)
            max_val = max(y_values)

            # Add buffer for better visualization
            range_val = max_val - min_val
            buffer = range_val * 0.1 if range_val > 0 else 0.5  # Ensure non-zero buffer

            min_bound = max(0, min_val - buffer) if min_val >= 0 else min_val - buffer
            max_bound = max_val + buffer

            return min_bound, max_bound
        except Exception as e:
            logger.error(f"Error calculating y-axis bounds: {str(e)}")
            return 0, 5

    def get_chart_specific_opts(self) -> dict:
        """Get chart type specific options. Override in subclasses."""
        y_min, y_max = self.get_y_axis_bounds()

        # Create basic options with improved responsiveness and label positioning
        return {
            "title_opts": opts.TitleOpts(
                title=self.options.get("title", ""),
                subtitle=self.options.get("subtitle", ""),
                title_textstyle_opts=opts.TextStyleOpts(
                    font_size=16,
                    font_weight="bold",
                ),
            ),
            "tooltip_opts": opts.TooltipOpts(
                trigger="axis",
                axis_pointer_type="cross",
                background_color="rgba(50,50,50,0.9)",
                border_width=0,
                textstyle_opts=opts.TextStyleOpts(color="#ffffff", font_size=12),
            ),
            "toolbox_opts": opts.ToolboxOpts(
                is_show=True,
                feature={
                    "dataZoom": {"yAxisIndex": "none"},
                    "restore": {},
                    "saveAsImage": {},
                },
                pos_left="right",
                orient="vertical",
            ),
            "xaxis_opts": opts.AxisOpts(
                type_="category",
                boundary_gap=True,
                axislabel_opts=opts.LabelOpts(
                    position="bottom",
                    rotate=30,
                    margin=10,
                    font_size=12,
                    is_show=True,
                ),
            ),
            "yaxis_opts": opts.AxisOpts(
                type_="value",
                min_=y_min,
                max_=y_max,
                axislabel_opts=opts.LabelOpts(font_size=12, is_show=True),
            ),
        }

    def initialize_chart(self, filtered_data: Optional[pd.DataFrame] = None) -> Chart:
        """Initialize the chart with common options."""
        chart_class = self.get_chart_class()
        if not chart_class:
            raise ValueError(f"Unknown chart type: {self.chart_details.chart_type}")

        # Store chart ID for cross-chart interactions
        self.chart_id = f"chart_{self.chart_details.id}"
        # Store for potential linked charts
        self._linked_charts: List[Chart] = []

        # Get chart options with responsive sizing
        width = cast(str, self.options.get("width", "auto"))
        height = cast(str, self.options.get("height", "600px"))
        theme = cast(str, self.options.get("theme", "white"))

        # Determine font size based on container width for responsiveness
        base_font_size = 12
        if width == "auto" or width == "100%":
            # Auto-responsive font size
            label_font_size = base_font_size
            title_font_size = base_font_size + 2
        elif isinstance(width, str) and width.endswith("px"):
            # Try to extract pixel width and adjust font size accordingly
            try:
                pixel_width = int(width.rstrip("px"))
                if pixel_width < 400:
                    label_font_size = (
                        base_font_size - 2
                    )  # Smaller font for small containers
                    title_font_size = base_font_size
                elif pixel_width > 1200:
                    label_font_size = (
                        base_font_size + 2
                    )  # Larger font for large containers
                    title_font_size = base_font_size + 4
                else:
                    label_font_size = base_font_size
                    title_font_size = base_font_size + 2
            except ValueError:
                label_font_size = base_font_size
                title_font_size = base_font_size + 2
        else:
            label_font_size = base_font_size
            title_font_size = base_font_size + 2

        # Store font sizes for use in other methods
        self._responsive_label_font_size = label_font_size
        self._responsive_title_font_size = title_font_size

        # Create chart instance with responsive options
        chart = chart_class(
            init_opts=opts.InitOpts(
                width=width,
                height=height,
                theme=theme,
                renderer="canvas",
                animation_opts=opts.AnimationOpts(animation=False),
            )
        )

        # Get chart options
        chart_opts = self.get_chart_specific_opts()

        # Enhance tooltip styling for better readability
        if "tooltip_opts" not in chart_opts:
            chart_opts["tooltip_opts"] = opts.TooltipOpts(
                trigger="axis",
                axis_pointer_type="cross",
                background_color="rgba(50,50,50,0.9)",  # Darker background
                border_color="#ccc",
                border_width=0,
                textstyle_opts=opts.TextStyleOpts(
                    color="#fff",  # White text for contrast
                    font_size=14,
                ),
            )

        chart.set_global_opts(**chart_opts)

        # Add responsive configuration
        chart.js_host = ""

        # Add additional initialization options for responsiveness
        if not hasattr(chart, "options") or not chart.options:
            chart.options = {}

        chart.options.update(
            {
                "animation": False,
                "grid": {
                    "top": "10%",
                    "right": "15%",
                    "bottom": "15%",
                    "left": "10%",
                    "containLabel": True,
                },
                # Accessibility options
                "aria": {
                    "show": True,
                    "description": getattr(self.chart_details, "title", None)
                    or "Chart",
                    "general": {
                        "withTitle": True,
                        "withDesc": True,
                    },
                    "label": {
                        "enabled": True,
                    },
                },
                "dataZoom": [
                    {
                        "type": "slider",
                        "show": True,
                        "realtime": True,
                        "start": 0,
                        "end": 100,
                        "bottom": "10%",
                        "xAxisIndex": [0],
                        "height": 20,
                    },
                    {"type": "inside", "xAxisIndex": [0], "start": 0, "end": 100},
                ],
            }
        )

        return chart

    def configure_chart(
        self, chart: Chart, filtered_data: Optional[pd.DataFrame] = None
    ) -> None:
        """Configure chart with data. This method handles common data processing for all chart types.

        Individual chart classes should override this method only if they need to change the basic
        data processing logic. Otherwise, they should override get_chart_specific_opts() and
        add_series_to_chart() to customize chart appearance.
        """
        if filtered_data is None:
            return

        # Process data based on chart type
        processed_data = self._process_data(filtered_data)

        # Get x-axis data
        x_axis_field = cast(DjangoFieldLike, self.options["x_axis_column"])
        x_field = x_axis_field.field_name
        x_axis_data = self._get_x_axis_data(processed_data, x_field)

        # Add x-axis
        chart.add_xaxis(x_axis_data)

        # Add series for each y-axis column
        for y_axis_column in self._get_y_axis_columns():
            field = y_axis_column["field"]
            field_name = field.field_name
            series_name = self._get_series_name(y_axis_column)

            # Get y values aligned with x-axis data
            y_values = self._get_y_values(
                processed_data, x_axis_data, x_field, field_name
            )

            # Add the series to the chart
            self.add_series_to_chart(
                chart=chart,
                series_name=series_name,
                y_values=y_values,
                color=y_axis_column.get("color"),
                value_mapping=y_axis_column.get("value_mapping"),
            )

    def _get_y_axis_columns(self) -> List[Dict[str, Any]]:
        """Get y-axis columns configuration."""
        y_axis_columns = self.options["y_axis_column"]
        if not isinstance(y_axis_columns, list):
            y_axis_columns = [y_axis_columns]
        return cast(List[Dict[str, Any]], y_axis_columns)

    def _get_series_name(self, y_axis_column: Dict[str, Any]) -> str:
        """Get series name from y-axis column configuration."""
        return str(y_axis_column.get("label") or y_axis_column["field"].field_name)

    def add_series_to_chart(
        self,
        chart: Chart,
        series_name: str,
        y_values: List[Any],
        color: Optional[str] = None,
        value_mapping: Optional[Dict[Any, Any]] = None,
    ) -> None:
        """Add a series to the chart with specific styling.

        This method can be overridden by subclasses to provide chart-specific styling.
        """
        # Add series to chart
        # This is a base implementation that will be overridden by subclasses
        if not hasattr(chart, "add_yaxis"):
            logger.warning("Chart does not have add_yaxis method")
            return

        # Get style options
        style_opts = self.get_series_style_opts(color)

        # Add label options to position labels at the bottom
        if "label_opts" not in style_opts:
            style_opts["label_opts"] = opts.LabelOpts(
                is_show=True,
                position="bottom",  # Position labels at the bottom
                font_size=12,
                font_weight="normal",
                color="#333",
            )

        # Add series to chart
        chart.add_yaxis(
            series_name=series_name,
            y_axis=y_values,
            **style_opts,
        )

    def get_series_style_opts(self, color: Optional[str] = None) -> Dict[str, Any]:
        """Get series-specific styling options.

        This method should be overridden by subclasses to provide series-specific styling options.
        """
        # Default options for all chart types
        return {
            "itemstyle_opts": opts.ItemStyleOpts(color=color) if color else None,
        }

    def add_series(self, chart: Chart, filtered_data: pd.DataFrame) -> Chart:
        """Add series data to the chart."""
        raise NotImplementedError("Subclasses must implement add_series")

    def _optimize_dataset(
        self, data: pd.DataFrame, max_points: int = 1000
    ) -> pd.DataFrame:
        """Optimize large datasets by sampling to improve performance.

        For large datasets, this method will sample the data to reduce the number of points
        while preserving the overall shape of the data.

        Args:
            data: The input DataFrame
            max_points: Maximum number of data points to display

        Returns:
            Optimized DataFrame with reduced number of points if necessary
        """
        if data is None or len(data) <= max_points:
            return data

        # Calculate sampling rate
        sample_rate = max_points / len(data)

        # Use systematic sampling to preserve data shape
        if "date" in data.columns or "time" in data.columns:
            # For time series data, preserve time distribution
            time_col = "date" if "date" in data.columns else "time"
            # Sort by time column
            data = data.sort_values(by=time_col)
            # Sample evenly across time
            indices = np.linspace(0, len(data) - 1, max_points).astype(int)
            return data.iloc[indices].copy()  # Return explicit DataFrame
        else:
            # For non-time series data, use random sampling
            return data.sample(n=max_points)

    def _handle_regular_data(self, chart: Chart, filtered_data: pd.DataFrame) -> None:
        """Handle non-time-based data."""
        # Optimize large datasets for performance
        optimized_data = self._optimize_dataset(filtered_data)

        # Get x-axis field name
        x_axis_field = cast(DjangoFieldLike, self.options["x_axis_column"])
        x_field = x_axis_field.field_name
        x_axis_data = optimized_data[x_field].tolist()

        # Sort values if needed
        sort_order = self.options.get("sort_order", "asc")
        x_axis_data = sorted(x_axis_data, reverse=(sort_order == "desc"))  # type: ignore[type-var]

        # Add x-axis data
        chart.add_xaxis(x_axis_data)

        # Get y-axis columns configuration
        y_axis_columns = self._get_y_axis_columns()

        # Add series for each y-axis column
        for y_axis_column in y_axis_columns:
            # Get y-axis field name
            y_field = cast(DjangoFieldLike, y_axis_column["field"])
            field_name = y_field.field_name
            y_values = filtered_data[field_name].tolist()

            # Get series name from configuration
            series_name = self._get_series_name(y_axis_column)

            # Get value mapping from configuration
            value_mapping = self._get_value_mapping(y_axis_column)

            # Add series to chart
            self.add_series_to_chart(
                chart=chart,
                series_name=series_name,
                y_values=y_values,
                color=y_axis_column.get("color"),
                value_mapping=value_mapping,
            )

    def _process_data(self, filtered_data: pd.DataFrame) -> pd.DataFrame:
        """Process data based on chart type and options.

        This method can be overridden by subclasses to perform chart-specific data processing.
        """
        # By default, just return the filtered data as is
        return filtered_data

    def _get_x_axis_data(self, processed_data: pd.DataFrame, x_field: str) -> List[Any]:
        """Get x-axis data from processed data."""
        # Extract x-axis values
        x_axis_data = processed_data[x_field].tolist()

        # Sort if needed
        sort_order = self.options.get("sort_order", "asc")
        return sorted(x_axis_data, reverse=(sort_order == "desc"))

    def _get_y_values(
        self,
        processed_data: pd.DataFrame,
        x_axis_data: List[Any],
        x_field: str,
        y_field: str,
    ) -> List[float]:
        """Get y-axis values aligned with x-axis data.

        Assumes that the data is already properly aggregated from SQL queries.
        Simply maps the values to ensure alignment with the x-axis order.
        """
        # Create a mapping from x values to their corresponding y values
        x_to_y_map = {}

        # Check if the y_field exists in the DataFrame
        if y_field not in processed_data.columns:
            logger.warning(f"Y-field '{y_field}' not found in processed data")
            return [0.0] * len(x_axis_data)

        # Get the column data directly
        y_column = processed_data[y_field]

        # Check if y_column is a DataFrame (can happen with SQL queries)
        if isinstance(y_column, pd.DataFrame):
            logger.debug(f"Y-column '{y_field}' is a DataFrame, using first column")
            if y_column.empty:
                return [0.0] * len(x_axis_data)
            y_column = y_column.iloc[:, 0]  # Take the first column

        # Create the mapping efficiently
        for idx, row in processed_data.iterrows():
            x_val = row[x_field]
            y_val = row[y_field]

            try:
                # Handle different data types efficiently
                if isinstance(y_val, pd.Series):
                    if not y_val.empty:
                        value = y_val.iloc[0]
                        if pd.notna(value):
                            x_to_y_map[x_val] = float(value)
                elif pd.notna(y_val):
                    x_to_y_map[x_val] = float(y_val)  # type: ignore
            except (ValueError, TypeError) as e:
                logger.warning(f"Error converting y-value for {x_val}: {e}")
            except Exception as e:
                logger.warning(f"Unexpected error processing y-value for {x_val}: {e}")

        # Create y_values array aligned with x_axis_data (list comprehension is more efficient)
        return [x_to_y_map.get(x_val, 0.0) for x_val in x_axis_data]

    def _get_value_mapping(self, y_axis_column: Dict[str, Any]) -> Dict[str, Any]:
        """Get value mapping from y-axis column configuration."""
        if not isinstance(y_axis_column, dict):
            return {}
        mapping = y_axis_column.get("value_mapping")
        return cast(Dict[str, Any], mapping) if mapping is not None else {}



"""Tests for authentication module."""

import unittest
from unittest.mock import MagicMock, patch

from dataspace_sdk.auth import AuthClient
from dataspace_sdk.exceptions import DataSpaceAuthError


class TestAuthClient(unittest.TestCase):
    """Test cases for AuthClient."""

    def setUp(self) -> None:
        """Set up test fixtures."""
        self.base_url = "https://api.test.com"
        self.auth_client = AuthClient(self.base_url)

    def test_init(self) -> None:
        """Test AuthClient initialization."""
        self.assertEqual(self.auth_client.base_url, self.base_url)
        self.assertIsNone(self.auth_client.access_token)
        self.assertIsNone(self.auth_client.refresh_token)
        self.assertIsNone(self.auth_client.user_info)

    @patch("dataspace_sdk.auth.requests.post")
    def test_login_success(self, mock_post: MagicMock) -> None:
        """Test successful login."""
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "access": "test_access_token",
            "refresh": "test_refresh_token",
            "user": {"id": "123", "username": "testuser"},
        }
        mock_post.return_value = mock_response

        result = self.auth_client.login_with_keycloak("test_keycloak_token")

        self.assertEqual(self.auth_client.access_token, "test_access_token")
        self.assertEqual(self.auth_client.refresh_token, "test_refresh_token")
        self.assertIsNotNone(self.auth_client.user_info)
        self.assertEqual(result["user"]["username"], "testuser")

    @patch("dataspace_sdk.auth.requests.post")
    def test_login_failure(self, mock_post: MagicMock) -> None:
        """Test failed login."""
        mock_response = MagicMock()
        mock_response.status_code = 401
        mock_response.json.return_value = {"error": "Invalid token"}
        mock_post.return_value = mock_response

        with self.assertRaises(DataSpaceAuthError):
            self.auth_client.login_with_keycloak("invalid_token")

    @patch("dataspace_sdk.auth.requests.post")
    def test_refresh_token_success(self, mock_post: MagicMock) -> None:
        """Test successful token refresh."""
        self.auth_client.refresh_token = "test_refresh_token"

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"access": "new_access_token"}
        mock_post.return_value = mock_response

        result = self.auth_client.refresh_access_token()

        self.assertEqual(result, "new_access_token")
        self.assertEqual(self.auth_client.access_token, "new_access_token")

    def test_refresh_token_no_refresh_token(self) -> None:
        """Test token refresh without refresh token."""
        with self.assertRaises(DataSpaceAuthError):
            self.auth_client.refresh_access_token()

    @patch("dataspace_sdk.auth.requests.get")
    def test_get_user_info_success(self, mock_get: MagicMock) -> None:
        """Test successful user info retrieval."""
        self.auth_client.access_token = "test_token"

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"user": {"id": "123", "username": "testuser"}}
        mock_get.return_value = mock_response

        result = self.auth_client.get_user_info()

        self.assertIsNotNone(result)
        self.assertEqual(result["user"]["username"], "testuser")

    def test_get_user_info_not_authenticated(self) -> None:
        """Test user info retrieval without authentication."""
        with self.assertRaises(DataSpaceAuthError):
            self.auth_client.get_user_info()

    def test_is_authenticated(self) -> None:
        """Test authentication status check."""
        self.assertFalse(self.auth_client.is_authenticated())

        self.auth_client.access_token = "test_token"
        self.assertTrue(self.auth_client.is_authenticated())


if __name__ == "__main__":
    unittest.main()



from enum import Enum
from typing import List, Optional

import strawberry
import strawberry_django
from strawberry import auto

from api.models import Metadata
from api.types.base_type import BaseType
from api.utils.enums import MetadataModels

metadata_models = strawberry.enum(MetadataModels)


@strawberry.enum
class ValidatorType(Enum):
    """Type of validator."""

    REGEX = "regex_validator"
    RANGE = "range_validator"
    MIN_LENGTH = "min_length_validator"


@strawberry_django.filter(Metadata)
class MetadataFilter:
    """Filter for metadata."""

    model: metadata_models
    enabled: bool


@strawberry_django.type(Metadata, fields="__all__", filters=MetadataFilter)
class TypeMetadata(BaseType):
    """Type for metadata."""

    id: auto
    label: auto
    model: metadata_models
    type: auto
    enabled: auto
    validator: List[ValidatorType]
    created: auto
    modified: auto



import uuid
from enum import Enum
from typing import Any, Dict, List, Optional, Union

import strawberry
import strawberry_django
from django.db.models import Q
from django.db.models.expressions import Combinable
from strawberry.types import Info

from api.models import (
    AccessModel,
    AccessModelResource,
    Dataset,
    Resource,
    ResourceSchema,
)
from api.types.type_access_model import TypeAccessModel
from api.utils.enums import AccessTypes

AccessTypesEnum = strawberry.enum(AccessTypes)  # type: ignore


@strawberry.input
class AccessModelResourceInput:
    resource: uuid.UUID
    fields: List[int]


@strawberry.input
class AccessModelInput:
    dataset: uuid.UUID
    name: str
    description: Optional[str]
    type: AccessTypesEnum
    resources: List[AccessModelResourceInput]


@strawberry.input
class EditAccessModelInput:
    access_model_id: Optional[uuid.UUID]
    dataset: uuid.UUID
    name: Optional[str]
    description: Optional[str]
    type: Optional[AccessTypesEnum]
    resources: Optional[List[AccessModelResourceInput]]


@strawberry.type(name="Query")
class Query:
    @strawberry_django.field
    def access_model_resources(
        self, info: Info, dataset_id: uuid.UUID
    ) -> List[TypeAccessModel]:
        models = AccessModel.objects.filter(dataset_id=dataset_id)
        return [TypeAccessModel.from_django(model) for model in models]

    @strawberry_django.field
    def access_model(self, info: Info, access_model_id: uuid.UUID) -> TypeAccessModel:
        model = AccessModel.objects.get(id=access_model_id)
        return TypeAccessModel.from_django(model)


def _add_resource_fields(
    access_model_resource: AccessModelResource,
    dataset_resource: Resource,
    fields: List[int],
) -> None:
    for field_id in fields:
        try:
            dataset_field = dataset_resource.resourceschema_set.get(id=field_id)
        except (Resource.DoesNotExist, ResourceSchema.DoesNotExist) as e:
            raise ValueError(f"Field with ID {field_id} does not exist.")
        access_model_resource.fields.add(dataset_field)
    access_model_resource.save()


def _add_update_access_model_resources(
    access_model: AccessModel,
    model_input_resources: Optional[List[AccessModelResourceInput]],
) -> None:
    if access_model.accessmodelresource_set.exists():
        access_model.accessmodelresource_set.all().delete()
        access_model.save()
    if not model_input_resources:
        return
    for resource_input in model_input_resources:
        try:
            dataset_resource = Resource.objects.get(id=resource_input.resource)
        except Resource.DoesNotExist as e:
            raise ValueError(
                f"Resource with ID {resource_input.resource} does not exist."
            )

        access_model_resource = AccessModelResource.objects.create(
            access_model=access_model, resource=dataset_resource
        )
        _add_resource_fields(
            access_model_resource, dataset_resource, resource_input.fields
        )


def _update_access_model_fields(
    access_model: AccessModel,
    access_model_input: Union[EditAccessModelInput, AccessModelInput],
) -> None:
    if hasattr(access_model_input, "name") and access_model_input.name:
        access_model.name = access_model_input.name
    if hasattr(access_model_input, "description"):
        access_model.description = access_model_input.description
    if hasattr(access_model_input, "type") and access_model_input.type:
        access_model.type = access_model_input.type
    access_model.save()


@strawberry.type
class Mutation:
    @strawberry_django.mutation(handle_django_errors=True)
    def create_access_model(
        self, info: Info, access_model_input: AccessModelInput
    ) -> TypeAccessModel:
        try:
            dataset = Dataset.objects.get(id=access_model_input.dataset)
        except Dataset.DoesNotExist:
            raise ValueError(
                f"Dataset with ID {access_model_input.dataset} does not exist."
            )

        access_model = AccessModel.objects.create(
            dataset=dataset,
            name=access_model_input.name,
            description=access_model_input.description,
            type=access_model_input.type.value,
        )

        _update_access_model_fields(access_model, access_model_input)
        _add_update_access_model_resources(access_model, access_model_input.resources)
        return TypeAccessModel.from_django(access_model)

    @strawberry_django.mutation(handle_django_errors=True)
    def edit_access_model(
        self, info: Info, access_model_input: EditAccessModelInput
    ) -> TypeAccessModel:
        if not access_model_input.access_model_id:
            try:
                dataset = Dataset.objects.get(id=access_model_input.dataset)
            except Dataset.DoesNotExist as e:
                raise ValueError(
                    f"Dataset with ID {access_model_input.dataset} does not exist."
                )
            access_model = AccessModel.objects.create(dataset=dataset)
        else:
            try:
                access_model = AccessModel.objects.get(
                    id=access_model_input.access_model_id
                )
            except AccessModel.DoesNotExist as e:
                raise ValueError(
                    f"Access Model with ID {access_model_input.access_model_id} does not exist."
                )

        _update_access_model_fields(access_model, access_model_input)
        _add_update_access_model_resources(access_model, access_model_input.resources)
        return TypeAccessModel.from_django(access_model)

    @strawberry_django.mutation(handle_django_errors=False)
    def delete_access_model(self, info: Info, access_model_id: uuid.UUID) -> bool:
        try:
            access_model = AccessModel.objects.get(id=access_model_id)
            access_model.delete()
            return True
        except AccessModel.DoesNotExist as e:
            raise ValueError(f"Access Model with ID {access_model_id} does not exist.")



"""Type stubs for django-cors-headers"""

from typing import Any, List, Optional, Union

class CorsModel:
    """Stub for CorsModel"""

    pass

class CorsMiddleware:
    """Stub for CorsMiddleware"""

    def __init__(self, get_response: Any) -> None: ...
    def __call__(self, request: Any) -> Any: ...

def get_model() -> CorsModel: ...
def signals_handler(
    sender: Any, instance: Any, created: bool, **kwargs: Any
) -> None: ...
def cors_allow_migrated_resources(
    request: Any, response: Any, *args: Any, **kwargs: Any
) -> None: ...
def check_errors(app_configs: Optional[List[Any]], **kwargs: Any) -> List[Any]: ...



import unittest
from unittest.mock import MagicMock

import pandas as pd

from api.types.charts.unified_chart import UnifiedChart


class MockResourceChartDetails:
    """Mock class for ResourceChartDetails to avoid Django dependency"""

    def __init__(
        self,
        chart_type="BAR",
        title="Test Chart",
        description="Test Description",
        options=None,
    ):
        self.chart_type = chart_type
        self.title = title
        self.description = description
        self.options = options or {}
        self.filters = []


class TestBarChart(unittest.TestCase):
    def setUp(self):
        """Set up test fixtures."""
        # Sample data for testing
        self.test_data = pd.DataFrame(
            {
                "category": ["A", "B", "C", "A", "B"],
                "value": [10, 20, 30, 15, 25],
                "time": ["2023-01", "2023-01", "2023-01", "2023-02", "2023-02"],
            }
        )

        # Mock field objects
        self.x_field = MagicMock()
        self.x_field.field_name = "category"

        self.y_field = MagicMock()
        self.y_field.field_name = "value"

        self.time_field = MagicMock()
        self.time_field.field_name = "time"

    def test_vertical_bar_chart(self):
        """Test vertical bar chart creation."""
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": [{"field": self.y_field, "label": "Value"}],
        }
        chart_details = MockResourceChartDetails(chart_type="BAR", options=options)
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        self.assertIsNotNone(result)
        self.assertEqual(result.options.get("xAxis")[0]["type"], "category")
        self.assertEqual(result.options.get("yAxis")[0]["type"], "value")

    def test_horizontal_bar_chart(self):
        """Test horizontal bar chart creation."""
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": [{"field": self.y_field, "label": "Value"}],
        }
        options_with_orientation = {**options, "orientation": "horizontal"}
        chart_details = MockResourceChartDetails(
            chart_type="BAR", options=options_with_orientation
        )
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        self.assertIsNotNone(result)
        self.assertEqual(result.options.get("xAxis")[0]["type"], "value")
        self.assertEqual(result.options.get("yAxis")[0]["type"], "category")

    def test_bar_styling(self):
        """Test bar chart styling options."""
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": [
                {"field": self.y_field, "label": "Value", "color": "#FF0000"}
            ],
        }
        chart_details = MockResourceChartDetails(chart_type="BAR", options=options)
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        series = result.options.get("series")[0]
        self.assertEqual(series.get("itemStyle").get("color"), "#FF0000")
        self.assertEqual(series.get("type"), "bar")

    def test_value_aggregation(self):
        """Test value aggregation in bar chart."""
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": [
                {"field": self.y_field, "label": "Value", "aggregate_type": "SUM"}
            ],
        }
        chart_details = MockResourceChartDetails(chart_type="BAR", options=options)
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        series = result.options.get("series")[0]
        data = series.get("data")
        # Check sum aggregation for category A (10 + 15)
        self.assertEqual(data[0].get("value"), 25)

    def test_axis_labels(self):
        """Test axis label configuration."""
        options = {
            "x_axis_column": self.x_field,
            "y_axis_column": [{"field": self.y_field, "label": "Custom Value Label"}],
            "x_axis_label": "Custom Category Label",
        }
        chart_details = MockResourceChartDetails(chart_type="BAR", options=options)
        print(chart_details)
        chart = UnifiedChart(chart_details, self.test_data)
        result = chart.create_chart()

        x_axis = result.options.get("xAxis")[0]
        y_axis = result.options.get("yAxis")[0]
        self.assertEqual(x_axis.get("name"), "Custom Category Label")
        self.assertEqual(y_axis.get("name"), "Custom Value Label")


if __name__ == "__main__":
    unittest.main()



from typing import Any

from django.db import models
from django.urls import reverse
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from api.utils.file_paths import _catalog_directory_path


class Catalog(models.Model):
    name = models.CharField(max_length=255)
    datasets = models.ManyToManyField("api.Dataset", related_name="catalogs")
    logo = models.ImageField(upload_to=_catalog_directory_path, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    homepage = models.URLField(blank=True)
    slug = models.SlugField(max_length=75, null=True, blank=False, unique=True)

    class Meta:
        verbose_name = _("catalog")
        verbose_name_plural = _("catalogs")

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("catalog_detail", kwargs={"pk": self.pk})

    def save(self, *args: Any, **kwargs: Any) -> None:
        self.slug = slugify(self.name)
        super().save(*args, **kwargs)



"""Type stubs for django-cors-headers.conf"""

from typing import Any, List, Optional, Set, Union

from django.conf import settings

CORS_ALLOW_ALL_ORIGINS: bool
CORS_ALLOWED_ORIGINS: List[str]
CORS_ALLOWED_ORIGIN_REGEXES: List[str]
CORS_ALLOW_METHODS: List[str]
CORS_ALLOW_HEADERS: List[str]
CORS_ALLOW_CREDENTIALS: bool
CORS_PREFLIGHT_MAX_AGE: int
CORS_EXPOSE_HEADERS: List[str]
CORS_URLS_REGEX: str



from typing import Optional

import strawberry
import strawberry_django
from django.db.models import Count, Q
from strawberry.types import Info

from api.models.Dataset import Dataset
from api.models.Organization import Organization
from api.models.UseCase import UseCase
from api.utils.enums import DatasetStatus, UseCaseStatus
from api.utils.graphql_telemetry import trace_resolver
from authorization.models import User


@strawberry.type
class StatsType:
    """Type for platform statistics"""

    total_users: int
    total_published_datasets: int
    total_publishers: int
    total_published_usecases: int
    total_organizations: int
    total_individuals: int


@strawberry.type
class Query:
    @strawberry_django.field
    @trace_resolver(name="stats", attributes={"component": "stats"})
    def stats(self, info: Info) -> StatsType:
        """Get platform statistics"""
        # Count total users
        total_users = User.objects.count()

        # Count published datasets
        total_published_datasets = Dataset.objects.filter(
            status=DatasetStatus.PUBLISHED
        ).count()

        # Count publishers (organizations and individuals who have published datasets)
        # First, get organizations that have published datasets
        org_publishers = (
            Organization.objects.filter(datasets__status=DatasetStatus.PUBLISHED)
            .distinct()
            .count()
        )

        # Then, get individual users who have published datasets
        individual_publishers = (
            User.objects.filter(
                datasets__status=DatasetStatus.PUBLISHED,
                datasets__organization__isnull=True,
            )
            .distinct()
            .count()
        )

        # Total publishers is the sum of organizations and individual publishers
        total_publishers = org_publishers + individual_publishers

        # Count published usecases
        total_published_usecases = UseCase.objects.filter(
            status=UseCaseStatus.PUBLISHED
        ).count()

        return StatsType(
            total_users=total_users,
            total_published_datasets=total_published_datasets,
            total_publishers=total_publishers,
            total_organizations=org_publishers,
            total_individuals=individual_publishers,
            total_published_usecases=total_published_usecases,
        )



import os
import subprocess
from typing import Any, Optional, cast

import structlog
from django.conf import settings
from django.core.management.base import BaseCommand

logger = structlog.getLogger(__name__)


class Command(BaseCommand):
    help = "Set up DVC repository for dataset versioning"

    def handle(self, *args: Any, **options: Any) -> None:
        try:
            repo_path = settings.DVC_REPO_PATH

            # Create directory if needed
            if not os.path.exists(repo_path):
                os.makedirs(repo_path)
                self.stdout.write(f"Created DVC repository directory at {repo_path}")

            # Initialize Git and DVC if not already done
            if not os.path.exists(os.path.join(repo_path, ".git")):
                subprocess.run(["git", "init"], cwd=repo_path, check=True)
                self.stdout.write("Initialized Git repository")

            if not os.path.exists(os.path.join(repo_path, ".dvc")):
                subprocess.run(["dvc", "init"], cwd=repo_path, check=True)
                self.stdout.write("Initialized DVC repository")

                # Configure chunking for large files
                subprocess.run(
                    ["dvc", "config", "cache.type", "hardlink,symlink"],
                    cwd=repo_path,
                    check=True,
                )
                subprocess.run(
                    ["dvc", "config", "cache.shared", "group"],
                    cwd=repo_path,
                    check=True,
                )

                # Configure cache size limits to prevent excessive disk usage
                subprocess.run(
                    ["dvc", "config", "cache.size_limit", "10G"],
                    cwd=repo_path,
                    check=True,
                )

                self.stdout.write("Configured DVC for large file handling")

            # Set up remote if configured
            if (
                hasattr(settings, "DVC_REMOTE_NAME")
                and hasattr(settings, "DVC_REMOTE_URL")
                and settings.DVC_REMOTE_NAME
                and settings.DVC_REMOTE_URL
            ):
                # Cast to str to satisfy mypy
                remote_name = cast(str, settings.DVC_REMOTE_NAME)
                remote_url = cast(str, settings.DVC_REMOTE_URL)

                subprocess.run(
                    [
                        "dvc",
                        "remote",
                        "add",
                        remote_name,
                        remote_url,
                    ],
                    cwd=repo_path,
                    check=True,
                )

                # Set as default remote
                subprocess.run(
                    ["dvc", "remote", "default", remote_name],
                    cwd=repo_path,
                    check=True,
                )

                self.stdout.write(f"Configured DVC remote: {remote_name}")

            # Set up Git user if not already configured
            try:
                # Check if Git user is configured
                result = subprocess.run(
                    ["git", "config", "user.name"],
                    cwd=repo_path,
                    capture_output=True,
                    text=True,
                )
                if not result.stdout.strip():
                    subprocess.run(
                        ["git", "config", "user.name", "DataEx Bot"],
                        cwd=repo_path,
                        check=True,
                    )
                    subprocess.run(
                        ["git", "config", "user.email", "dataex@example.com"],
                        cwd=repo_path,
                        check=True,
                    )
                    self.stdout.write("Configured Git user for DVC repository")
            except subprocess.CalledProcessError:
                # If checking fails, set the user anyway
                subprocess.run(
                    ["git", "config", "user.name", "DataEx Bot"],
                    cwd=repo_path,
                    check=True,
                )
                subprocess.run(
                    ["git", "config", "user.email", "dataex@example.com"],
                    cwd=repo_path,
                    check=True,
                )
                self.stdout.write("Configured Git user for DVC repository")

            self.stdout.write(self.style.SUCCESS("DVC repository set up successfully"))

        except Exception as e:
            import traceback

            logger.error(
                f"Failed to set up DVC repository: {str(e)}, Traceback: {traceback.format_exc()}"
            )
            self.stdout.write(
                self.style.ERROR(f"Failed to set up DVC repository: {str(e)}")
            )



import strawberry_django

from api.models import AccessModelResource
from api.types.type_resource import TypeResource, TypeResourceSchema


@strawberry_django.type(AccessModelResource, fields="__all__")
class TypeAccessModelResource:
    resource: TypeResource
    fields: list[TypeResourceSchema]



from typing import Any, List, Optional, cast

import strawberry
import structlog
from django.conf import settings
from strawberry.permission import BasePermission
from strawberry.types import Info

from api.utils.debug_utils import debug_context
from authorization.models import DatasetPermission, OrganizationMembership, User
from authorization.services import AuthorizationService

logger = structlog.getLogger(__name__)


class AllowAny(BasePermission):
    """
    Permission class that allows any access.
    This is the default permission class for GraphQL queries.
    """

    message = ""  # No message needed since permission is always granted

    def has_permission(self, source: Any, info: Info, **kwargs: Any) -> bool:
        return True


class IsAuthenticated(BasePermission):
    """
    Permission class that checks if the user is authenticated.

    """

    message = "User is not authenticated"

    def has_permission(self, source: Any, info: Info, **kwargs: Any) -> bool:

        # Debug the context to understand its structure
        debug_context(info, "Auth")

        # Log the type of the context object
        logger.info(f"Context type: {type(info.context).__name__}")

        # Get the request from context
        request = info.context

        # Log authentication status
        if hasattr(request, "user"):
            logger.info(f"User authenticated: {request.user.is_authenticated}")
            return request.user.is_authenticated  # type: ignore[no-any-return]
        else:
            logger.warning("No user attribute found in request")
            return False


class IsOrganizationMember(BasePermission):
    """
    Permission class that checks if the user is a member of the organization.

    """

    message = "User is not a member of the organization"

    def __init__(self, organization_id_arg: str = "organization_id"):
        self.organization_id_arg = organization_id_arg

    def has_permission(self, source: Any, info: Info, **kwargs: Any) -> bool:

        # Log the type of the context object
        logger.info(
            f"Context type in IsOrganizationMember: {type(info.context).__name__}"
        )

        request = info.context

        # If the user is a superuser, grant permission
        if request.user.is_superuser:
            return True

        # Get the organization ID from the arguments
        organization_id = kwargs.get(self.organization_id_arg)
        if not organization_id:
            return False

        # Check if the user is a member of the organization
        auth_service = AuthorizationService()
        # Use the check_organization_permission method instead
        return auth_service.check_organization_permission(
            request.user.id, organization_id, "view"
        )


class HasOrganizationRole(BasePermission):
    """
    Permission class that checks if the user has the required role in the organization.

    """

    message = "User does not have the required role in the organization"

    def __init__(
        self, operation: str = "view", organization_id_arg: str = "organization_id"
    ):
        # Convert operation to a list of role names for backward compatibility
        self.operation = operation
        self.organization_id_arg = organization_id_arg

    def has_permission(self, source: Any, info: Info, **kwargs: Any) -> bool:

        # Log the type of the context object
        logger.info(
            f"Context type in HasOrganizationRole: {type(info.context).__name__}"
        )

        request = info.context

        # If the user is a superuser, grant permission
        if request.user.is_superuser:
            return True

        # Get the organization ID from the arguments
        organization_id = kwargs.get(self.organization_id_arg)
        if not organization_id:
            return False

        # Check if the user has the required permission in the organization
        return AuthorizationService.check_organization_permission(
            request.user.id, organization_id, self.operation
        )  # type: ignore[no-any-return]

        return False


class HasDatasetPermission(BasePermission):
    """
    Permission class that checks if the user has the required permission for the dataset.

    """

    message = "User does not have the required permission for the dataset"

    def __init__(self, operation: str = "view", dataset_id_arg: str = "dataset_id"):
        self.operation = operation
        self.dataset_id_arg = dataset_id_arg

    def has_permission(self, source: Any, info: Info, **kwargs: Any) -> bool:

        # Log the type of the context object
        logger.info(
            f"Context type in HasDatasetPermission: {type(info.context).__name__}"
        )

        request = info.context

        # If the user is a superuser, grant permission
        if request.user.is_superuser:
            return True

        # Get the dataset ID from the arguments
        dataset_id = kwargs.get(self.dataset_id_arg)
        if not dataset_id:
            return False

        # Check if the user has the required permission for the dataset
        return AuthorizationService.check_dataset_permission(
            request.user.id, dataset_id, self.operation
        )  # type: ignore[no-any-return]



"""Custom middleware to handle CORS for OPTIONS requests."""

from typing import Any, Callable

from django.http import HttpRequest, HttpResponse


class OptionsCorsMiddleware:
    """Middleware that handles OPTIONS requests and adds CORS headers."""

    def __init__(self, get_response: Callable[[HttpRequest], HttpResponse]) -> None:
        self.get_response = get_response

    def __call__(self, request: HttpRequest) -> HttpResponse:
        # Handle OPTIONS requests directly
        if request.method == "OPTIONS":
            response = HttpResponse()
            response["Access-Control-Allow-Origin"] = "*"
            response["Access-Control-Allow-Methods"] = "GET, POST, OPTIONS, DELETE, PUT"
            response["Access-Control-Allow-Headers"] = (
                "DNT, User-Agent, X-Requested-With, If-Modified-Since, Cache-Control, "
                "Content-Type, Range, Authorization, x-keycloak-token, organization, "
                "dataspace, token"
            )
            response["Access-Control-Max-Age"] = "86400"
            return response

        # For non-OPTIONS requests, proceed normally
        response = self.get_response(request)

        # Ensure CORS headers are also added to regular responses
        response["Access-Control-Allow-Origin"] = "*"
        response["Access-Control-Allow-Methods"] = "GET, POST, OPTIONS, DELETE, PUT"

        return response



# Keycloak settings for Django integration

import os

from decouple import config

# Keycloak server settings
KEYCLOAK_SERVER_URL = config(
    "KEYCLOAK_SERVER_URL", default="http://localhost:8080/auth/"
)
KEYCLOAK_REALM = config("KEYCLOAK_REALM", default="dataexchange")
KEYCLOAK_CLIENT_ID = config("KEYCLOAK_CLIENT_ID", default="dataexchange-backend")
KEYCLOAK_CLIENT_SECRET = config("KEYCLOAK_CLIENT_SECRET", default="")

# Keycloak client settings
KEYCLOAK_CLIENT_PUBLIC_KEY = config("KEYCLOAK_CLIENT_PUBLIC_KEY", default="")
KEYCLOAK_CLIENT_SERVER_URL = config(
    "KEYCLOAK_CLIENT_SERVER_URL", default=KEYCLOAK_SERVER_URL
)
KEYCLOAK_CLIENT_REALM = config("KEYCLOAK_CLIENT_REALM", default=KEYCLOAK_REALM)
KEYCLOAK_CLIENT_CLIENT_ID = config(
    "KEYCLOAK_CLIENT_CLIENT_ID", default=KEYCLOAK_CLIENT_ID
)

# Keycloak admin credentials for user management
KEYCLOAK_ADMIN_USERNAME = config("KEYCLOAK_ADMIN_USERNAME", default="")
KEYCLOAK_ADMIN_PASSWORD = config("KEYCLOAK_ADMIN_PASSWORD", default="")

# Keycloak authorization settings
KEYCLOAK_AUTHORIZATION_CONFIG = {
    "KEYCLOAK_SERVER_URL": KEYCLOAK_SERVER_URL,
    "KEYCLOAK_REALM": KEYCLOAK_REALM,
    "KEYCLOAK_CLIENT_ID": KEYCLOAK_CLIENT_ID,
    "KEYCLOAK_CLIENT_SECRET": KEYCLOAK_CLIENT_SECRET,
}

# JWT Authentication settings
SIMPLE_JWT = {
    "AUTH_HEADER_TYPES": ("Bearer",),
    "USER_ID_FIELD": "id",
    "USER_ID_CLAIM": "user_id",
    "AUTH_TOKEN_CLASSES": ("rest_framework_simplejwt.tokens.AccessToken",),
    "TOKEN_TYPE_CLAIM": "token_type",
}

# Django REST Framework JWT settings
REST_FRAMEWORK_JWT = {
    "JWT_AUTH_HEADER_PREFIX": "Bearer",
    "JWT_ALGORITHM": "RS256",
    "JWT_AUDIENCE": KEYCLOAK_CLIENT_ID,
    "JWT_ISSUER": f"{KEYCLOAK_SERVER_URL}realms/{KEYCLOAK_REALM}",
    "JWT_AUTH_COOKIE": "jwt-auth",
}



"""
Root conftest.py for pytest configuration
"""

import os
import sys

import django
import pytest  # type: ignore
from django.conf import settings
from django.db.backends.base.base import BaseDatabaseWrapper

# Add the project root directory to Python path
sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))


@pytest.fixture(autouse=True)
def enable_db_access_for_all_tests(db: BaseDatabaseWrapper) -> None:
    """
    Global fixture that enables database access for all tests
    """
    pass



"""Example: Error handling and token refresh."""

from dataspace_sdk import (
    DataSpaceAPIError,
    DataSpaceAuthError,
    DataSpaceClient,
    DataSpaceNotFoundError,
    DataSpaceValidationError,
)

# Initialize client
client = DataSpaceClient(base_url="https://api.dataspace.example.com")
keycloak_token = "your_keycloak_token_here"

# Login with error handling
try:
    user_info = client.login(keycloak_token=keycloak_token)
    print(f"✓ Logged in as: {user_info['user']['username']}")
except DataSpaceAuthError as e:
    print(f"✗ Authentication failed: {e.message}")
    print(f"  Status code: {e.status_code}")
    exit(1)

# Get dataset with error handling
dataset_id = "550e8400-e29b-41d4-a716-446655440000"

try:
    dataset = client.datasets.get_by_id(dataset_id)
    print(f"\n✓ Found dataset: {dataset['title']}")

except DataSpaceNotFoundError as e:
    print(f"\n✗ Dataset not found: {e.message}")
    print(f"  Dataset ID: {dataset_id}")

except DataSpaceAuthError as e:
    print(f"\n✗ Authentication error: {e.message}")
    print("  Attempting to refresh token...")

    try:
        new_token = client.refresh_token()
        print("  ✓ Token refreshed successfully")

        # Retry the request
        dataset = client.datasets.get_by_id(dataset_id)
        print(f"  ✓ Found dataset: {dataset['title']}")

    except DataSpaceAuthError as refresh_error:
        print(f"  ✗ Token refresh failed: {refresh_error.message}")
        print("  Please login again")

except DataSpaceValidationError as e:
    print(f"\n✗ Validation error: {e.message}")
    print(f"  Details: {e.response}")

except DataSpaceAPIError as e:
    print(f"\n✗ API error: {e.message}")
    print(f"  Status code: {e.status_code}")
    print(f"  Response: {e.response}")

# Search with error handling
try:
    results = client.datasets.search(
        query="test", status="INVALID_STATUS"  # This might cause a validation error
    )
    print(f"\n✓ Found {results['total']} datasets")

except DataSpaceValidationError as e:
    print(f"\n✗ Invalid search parameters: {e.message}")
    print(f"  Response: {e.response}")

except DataSpaceAPIError as e:
    print(f"\n✗ Search failed: {e.message}")

# Check authentication status
if client.is_authenticated():
    print("\n✓ Client is authenticated")
    if client.user:
        print(f"  User: {client.user['username']}")
        print(f"  Organizations: {len(client.user['organizations'])}")
else:
    print("\n✗ Client is not authenticated")

# Graceful error handling for network issues
try:
    # This will fail if the API is unreachable
    trending = client.datasets.get_trending(limit=5)
    if trending and "results" in trending:
        print(f"\n✓ Found {len(trending['results'])} trending datasets")

except DataSpaceAPIError as e:
    if "Network error" in str(e):
        print("\n✗ Network error: Unable to reach the API")
        print("  Please check your internet connection and API URL")
    else:
        print(f"\n✗ API error: {e.message}")



CACHES = {
    "default": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://redis:6379/1",  # Using redis container hostname
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
            "SOCKET_CONNECT_TIMEOUT": 5,
            "SOCKET_TIMEOUT": 5,
            "COMPRESSOR": "django_redis.compressors.zlib.ZlibCompressor",
            "IGNORE_EXCEPTIONS": True,
            "RETRY_ON_TIMEOUT": True,  # Added retry on timeout
            "CONNECTION_POOL_KWARGS": {
                "max_connections": 100
            },  # Added connection pool settings
        },
    }
}

# Cache time to live is 15 minutes.
CACHE_TTL = 60 * 15

# Key prefix for cache keys
CACHE_KEY_PREFIX = "dataex"

# Session cache configuration
SESSION_ENGINE = "django.contrib.sessions.backends.cache"
SESSION_CACHE_ALIAS = "default"

# Cache middleware settings
CACHE_MIDDLEWARE_SECONDS = 60 * 5  # 5 minutes
CACHE_MIDDLEWARE_KEY_PREFIX = "dataex"



"""Tests for base API client."""

import unittest
from unittest.mock import MagicMock, patch

from dataspace_sdk.base import BaseAPIClient
from dataspace_sdk.exceptions import (
    DataSpaceAPIError,
    DataSpaceAuthError,
    DataSpaceNotFoundError,
    DataSpaceValidationError,
)


class TestBaseAPIClient(unittest.TestCase):
    """Test cases for BaseAPIClient."""

    def setUp(self) -> None:
        """Set up test fixtures."""
        self.base_url = "https://api.test.com"
        self.client = BaseAPIClient(self.base_url)

    def test_init(self) -> None:
        """Test BaseAPIClient initialization."""
        self.assertEqual(self.client.base_url, self.base_url)
        self.assertIsNone(self.client.auth_client)

    def test_get_headers_no_auth(self) -> None:
        """Test header generation without authentication."""
        headers = self.client._get_headers()
        self.assertEqual(headers["Content-Type"], "application/json")
        self.assertNotIn("Authorization", headers)

    def test_get_headers_with_auth(self) -> None:
        """Test header generation with authentication."""
        mock_auth = MagicMock()
        mock_auth.is_authenticated.return_value = True
        mock_auth.access_token = "test_token"
        self.client.auth_client = mock_auth

        headers = self.client._get_headers()
        self.assertEqual(headers["Authorization"], "Bearer test_token")

    @patch("dataspace_sdk.base.requests.request")
    def test_make_request_success_200(self, mock_request: MagicMock) -> None:
        """Test successful request with 200 status."""
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.content = b'{"data": "test"}'
        mock_response.json.return_value = {"data": "test"}
        mock_request.return_value = mock_response

        result = self.client._make_request("GET", "/test")
        self.assertEqual(result["data"], "test")

    @patch("dataspace_sdk.base.requests.request")
    def test_make_request_success_204(self, mock_request: MagicMock) -> None:
        """Test successful request with 204 status."""
        mock_response = MagicMock()
        mock_response.status_code = 204
        mock_request.return_value = mock_response

        result = self.client._make_request("DELETE", "/test")
        self.assertEqual(result, {})

    @patch("dataspace_sdk.base.requests.request")
    def test_make_request_401_error(self, mock_request: MagicMock) -> None:
        """Test request with 401 authentication error."""
        mock_response = MagicMock()
        mock_response.status_code = 401
        mock_response.content = b'{"error": "Unauthorized"}'
        mock_response.json.return_value = {"error": "Unauthorized"}
        mock_request.return_value = mock_response

        with self.assertRaises(DataSpaceAuthError):
            self.client._make_request("GET", "/test")

    @patch("dataspace_sdk.base.requests.request")
    def test_make_request_404_error(self, mock_request: MagicMock) -> None:
        """Test request with 404 not found error."""
        mock_response = MagicMock()
        mock_response.status_code = 404
        mock_response.content = b'{"error": "Not found"}'
        mock_response.json.return_value = {"error": "Not found"}
        mock_request.return_value = mock_response

        with self.assertRaises(DataSpaceNotFoundError):
            self.client._make_request("GET", "/test")

    @patch("dataspace_sdk.base.requests.request")
    def test_make_request_400_error(self, mock_request: MagicMock) -> None:
        """Test request with 400 validation error."""
        mock_response = MagicMock()
        mock_response.status_code = 400
        mock_response.content = b'{"error": "Validation failed"}'
        mock_response.json.return_value = {"error": "Validation failed"}
        mock_request.return_value = mock_response

        with self.assertRaises(DataSpaceValidationError):
            self.client._make_request("GET", "/test")

    @patch("dataspace_sdk.base.requests.request")
    def test_make_request_500_error(self, mock_request: MagicMock) -> None:
        """Test request with 500 server error."""
        mock_response = MagicMock()
        mock_response.status_code = 500
        mock_response.content = b'{"error": "Server error"}'
        mock_response.json.return_value = {"error": "Server error"}
        mock_request.return_value = mock_response

        with self.assertRaises(DataSpaceAPIError):
            self.client._make_request("GET", "/test")

    def test_get_method(self) -> None:
        """Test GET method wrapper."""
        with patch.object(self.client, "_make_request") as mock_request:
            mock_request.return_value = {"data": "test"}
            result = self.client.get("/test", params={"key": "value"})
            mock_request.assert_called_once_with("GET", "/test", params={"key": "value"})
            self.assertEqual(result["data"], "test")

    def test_post_method(self) -> None:
        """Test POST method wrapper."""
        with patch.object(self.client, "_make_request") as mock_request:
            mock_request.return_value = {"data": "test"}
            result = self.client.post("/test", json_data={"key": "value"})
            mock_request.assert_called_once_with(
                "POST", "/test", data=None, json_data={"key": "value"}
            )
            self.assertEqual(result["data"], "test")


if __name__ == "__main__":
    unittest.main()



from typing import Optional

import strawberry
import strawberry_django
from strawberry import auto

from api.models import Geography
from api.types.base_type import BaseType


@strawberry_django.type(Geography)
class TypeGeo(BaseType):
    id: auto
    name: auto
    code: auto
    type: auto

    @strawberry.field(description="Parent geography")
    def parent_id(self) -> Optional["TypeGeo"]:
        """Get parent geography."""
        if self.parent_id:  # type: ignore
            return TypeGeo.from_django(self.parent_id)  # type: ignore
        return None



import strawberry_django

from api.models import DatasetMetadata
from api.types.base_type import BaseType
from api.types.type_metadata import TypeMetadata


@strawberry_django.type(DatasetMetadata, fields="__all__")
class TypeDatasetMetadata(BaseType):
    metadata_item: TypeMetadata



"""Tests for AI model resource client."""

import unittest
from unittest.mock import MagicMock, patch

from dataspace_sdk.resources.aimodels import AIModelClient


class TestAIModelClient(unittest.TestCase):
    """Test cases for AIModelClient."""

    def setUp(self) -> None:
        """Set up test fixtures."""
        self.base_url = "https://api.test.com"
        self.auth_client = MagicMock()
        self.client = AIModelClient(self.base_url, self.auth_client)

    def test_init(self) -> None:
        """Test AIModelClient initialization."""
        self.assertEqual(self.client.base_url, self.base_url)
        self.assertEqual(self.client.auth_client, self.auth_client)

    @patch.object(AIModelClient, "get")
    def test_search_models(self, mock_get: MagicMock) -> None:
        """Test AI model search."""
        mock_get.return_value = {
            "total": 5,
            "results": [{"id": "1", "displayName": "Test Model", "modelType": "LLM"}],
        }

        result = self.client.search(query="test", page=1, page_size=10)

        self.assertEqual(result["total"], 5)
        self.assertEqual(len(result["results"]), 1)
        self.assertEqual(result["results"][0]["displayName"], "Test Model")
        mock_get.assert_called_once()

    @patch.object(AIModelClient, "get")
    def test_get_model_by_id(self, mock_get: MagicMock) -> None:
        """Test get AI model by ID."""
        mock_get.return_value = {
            "id": "123",
            "displayName": "Test Model",
            "modelType": "LLM",
            "provider": "OpenAI",
        }

        result = self.client.get_by_id("123")

        self.assertEqual(result["id"], "123")
        self.assertEqual(result["displayName"], "Test Model")
        mock_get.assert_called_once()

    @patch.object(AIModelClient, "post")
    def test_get_model_by_id_graphql(self, mock_post: MagicMock) -> None:
        """Test get AI model by ID using GraphQL."""
        mock_post.return_value = {
            "data": {
                "aiModel": {
                    "id": "123",
                    "displayName": "Test Model",
                    "description": "A test model",
                }
            }
        }

        result = self.client.get_by_id_graphql("123")

        self.assertEqual(result["id"], "123")
        self.assertEqual(result["displayName"], "Test Model")
        mock_post.assert_called_once()

    @patch.object(AIModelClient, "post")
    def test_list_all_models(self, mock_post: MagicMock) -> None:
        """Test list all AI models."""
        mock_post.return_value = {"data": {"aiModels": [{"id": "1", "displayName": "Model 1"}]}}

        result = self.client.list_all(limit=10, offset=0)

        self.assertIsInstance(result, (list, dict))
        mock_post.assert_called_once()

    @patch.object(AIModelClient, "post")
    def test_get_organization_models(self, mock_post: MagicMock) -> None:
        """Test get organization AI models."""
        mock_post.return_value = {"data": {"aiModels": [{"id": "1", "name": "Org Model"}]}}

        result = self.client.get_organization_models("org-123", limit=10)

        self.assertIsInstance(result, (list, dict))
        mock_post.assert_called_once()

    @patch.object(AIModelClient, "get")
    def test_search_with_filters(self, mock_get: MagicMock) -> None:
        """Test AI model search with filters."""
        mock_get.return_value = {"total": 3, "results": []}

        result = self.client.search(
            query="language",
            tags=["nlp"],
            sectors=["tech"],
            model_type="LLM",
            provider="OpenAI",
            status="ACTIVE",
        )

        self.assertEqual(result["total"], 3)
        mock_get.assert_called_once()

    @patch.object(AIModelClient, "post")
    def test_graphql_error_handling(self, mock_post: MagicMock) -> None:
        """Test GraphQL error handling."""
        from dataspace_sdk.exceptions import DataSpaceAPIError

        mock_post.return_value = {"errors": [{"message": "GraphQL error"}]}

        with self.assertRaises(DataSpaceAPIError):
            self.client.get_by_id_graphql("123")


if __name__ == "__main__":
    unittest.main()



from typing import Any, Dict, List, Optional

import structlog
from django.conf import settings
from django.db import transaction
from keycloak import KeycloakOpenID
from keycloak.exceptions import KeycloakError

from api.models import Organization
from authorization.models import OrganizationMembership, User

logger = structlog.getLogger(__name__)


class KeycloakManager:
    """
    Utility class to manage Keycloak integration with Django.
    Handles token validation, user synchronization, and role mapping.
    """

    def __init__(self) -> None:
        self.server_url = settings.KEYCLOAK_SERVER_URL
        self.realm = settings.KEYCLOAK_REALM
        self.client_id = settings.KEYCLOAK_CLIENT_ID
        self.client_secret = settings.KEYCLOAK_CLIENT_SECRET

        self.keycloak_openid = KeycloakOpenID(
            server_url=self.server_url,
            client_id=self.client_id,
            realm_name=self.realm,
            client_secret_key=self.client_secret,
        )

    def get_keycloak_client(self) -> KeycloakOpenID:
        """
        Get a Keycloak client instance.
        """
        return self.keycloak_openid

    def get_token(self, username: str, password: str) -> Dict[str, Any]:
        """
        Get a Keycloak token for a user.

        Args:
            username: The username
            password: The password

        Returns:
            Dict containing the token information
        """
        try:
            return self.keycloak_openid.token(username, password)
        except KeycloakError as e:
            logger.error(f"Error getting token: {e}")
            raise

    def validate_token(self, token: str) -> Dict[str, Any]:
        """
        Validate a Keycloak token and return the user info.

        Args:
            token: The token to validate

        Returns:
            Dict containing the user information
        """
        try:
            # Verify the token is valid
            token_info = self.keycloak_openid.introspect(token)
            if not token_info.get("active", False):
                logger.warning("Token is not active")
                return {}

            # Get user info from the token
            user_info = self.keycloak_openid.userinfo(token)
            return user_info
        except KeycloakError as e:
            logger.error(f"Error validating token: {e}")
            return {}

    def get_user_roles(self, token_info: dict) -> list[str]:
        """
        Extract roles from a Keycloak token.
        """
        roles: list[str] = []

        # Extract realm roles
        realm_access = token_info.get("realm_access", {})
        if realm_access and "roles" in realm_access:
            roles.extend(realm_access["roles"])  # type: ignore[no-any-return]

        # Extract client roles
        resource_access = token_info.get("resource_access", {})
        client_id = settings.KEYCLOAK_CLIENT_ID
        if resource_access and client_id in resource_access:
            client_roles = resource_access[client_id].get("roles", [])
            roles.extend(client_roles)

        return roles

    def get_user_organizations(self, token: str) -> List[Dict[str, Any]]:
        """
        Get the organizations a user belongs to from their token.
        This assumes that organization information is stored in the token
        as client roles or in user attributes.

        Args:
            token: The user's token

        Returns:
            List of organization information
        """
        try:
            # Decode the token to get user info
            token_info = self.keycloak_openid.decode_token(token)

            # Get organization info from resource_access or attributes
            # This implementation depends on how organizations are represented in Keycloak
            # This is a simplified example - adjust based on your Keycloak configuration
            resource_access = token_info.get("resource_access", {})
            client_roles = resource_access.get(self.client_id, {}).get("roles", [])

            # Extract organization info from roles
            # Format could be 'org_<org_id>_<role>' or similar
            organizations = []
            for role in client_roles:
                if role.startswith("org_"):
                    parts = role.split("_")
                    if len(parts) >= 3:
                        org_id = parts[1]
                        role_name = parts[2]
                        organizations.append(
                            {"organization_id": org_id, "role": role_name}
                        )

            return organizations
        except KeycloakError as e:
            logger.error(f"Error getting user organizations: {e}")
            return []

    @transaction.atomic
    def sync_user_from_keycloak(
        self,
        user_info: Dict[str, Any],
        roles: List[str],
        organizations: List[Dict[str, Any]],
    ) -> Optional[User]:
        """
        Synchronize user information from Keycloak to Django.
        Creates or updates the User and UserOrganization records.

        Args:
            user_info: User information from Keycloak
            roles: User roles from Keycloak
            organizations: User organization memberships from Keycloak

        Returns:
            The synchronized User object or None if failed
        """
        try:
            keycloak_id = user_info.get("sub")
            email = user_info.get("email")
            username = user_info.get("preferred_username") or email

            if not keycloak_id or not username:
                logger.error("Missing required user information from Keycloak")
                return None

            # Get or create the user
            user, created = User.objects.update_or_create(
                keycloak_id=keycloak_id,
                defaults={
                    "username": username,
                    "email": email,
                    "first_name": user_info.get("given_name", ""),
                    "last_name": user_info.get("family_name", ""),
                    "is_active": True,
                },
            )

            # Update user roles based on Keycloak roles
            if "admin" in roles:
                user.is_staff = True
                user.is_superuser = True
            else:
                user.is_staff = False
                user.is_superuser = False

            user.save()

            # Update organization memberships
            # First, get all existing organization memberships
            existing_memberships = OrganizationMembership.objects.filter(user=user)
            existing_org_ids = {
                membership.organization_id for membership in existing_memberships  # type: ignore[attr-defined]
            }

            # Process organizations from Keycloak
            for org_info in organizations:
                org_id = org_info.get("organization_id")
                role = org_info.get(
                    "role", "viewer"
                )  # Default to viewer if role not specified

                # Try to get the organization
                try:
                    organization = Organization.objects.get(id=org_id)  # type: ignore[misc]

                    # Create or update the membership
                    OrganizationMembership.objects.update_or_create(
                        user=user, organization=organization, defaults={"role": role}
                    )

                    # Remove from the set of existing memberships
                    if org_id in existing_org_ids:
                        existing_org_ids.remove(org_id)
                except Organization.DoesNotExist:
                    logger.warning(f"Organization with ID {org_id} does not exist")

            # Remove memberships that no longer exist in Keycloak
            if existing_org_ids:
                OrganizationMembership.objects.filter(
                    user=user, organization_id__in=existing_org_ids
                ).delete()

            return user
        except Exception as e:
            logger.error(f"Error synchronizing user from Keycloak: {e}")
            return None


# Create a singleton instance
keycloak_manager = KeycloakManager()



"""Tests for dataset resource client."""

import unittest
from unittest.mock import MagicMock, patch

from dataspace_sdk.resources.datasets import DatasetClient


class TestDatasetClient(unittest.TestCase):
    """Test cases for DatasetClient."""

    def setUp(self) -> None:
        """Set up test fixtures."""
        self.base_url = "https://api.test.com"
        self.auth_client = MagicMock()
        self.client = DatasetClient(self.base_url, self.auth_client)

    def test_init(self) -> None:
        """Test DatasetClient initialization."""
        self.assertEqual(self.client.base_url, self.base_url)
        self.assertEqual(self.client.auth_client, self.auth_client)

    @patch.object(DatasetClient, "get")
    def test_search_datasets(self, mock_get: MagicMock) -> None:
        """Test dataset search."""
        mock_get.return_value = {
            "total": 10,
            "results": [{"id": "1", "title": "Test Dataset"}],
        }

        result = self.client.search(query="test", page=1, page_size=10)

        self.assertEqual(result["total"], 10)
        self.assertEqual(len(result["results"]), 1)
        mock_get.assert_called_once()

    @patch.object(DatasetClient, "post")
    def test_get_dataset_by_id(self, mock_post: MagicMock) -> None:
        """Test get dataset by ID."""
        mock_post.return_value = {"data": {"dataset": {"id": "123", "title": "Test Dataset"}}}

        result = self.client.get_by_id("123")

        self.assertEqual(result["id"], "123")
        self.assertEqual(result["title"], "Test Dataset")

    @patch.object(DatasetClient, "post")
    def test_list_all_datasets(self, mock_post: MagicMock) -> None:
        """Test list all datasets."""
        mock_post.return_value = {"data": {"datasets": [{"id": "1", "title": "Dataset 1"}]}}

        result = self.client.list_all(limit=10, offset=0)

        self.assertIsInstance(result, (list, dict))

    @patch.object(DatasetClient, "get")
    def test_get_trending_datasets(self, mock_get: MagicMock) -> None:
        """Test get trending datasets."""
        mock_get.return_value = {"results": [{"id": "1", "title": "Trending Dataset"}]}

        result = self.client.get_trending(limit=5)

        self.assertIn("results", result)
        mock_get.assert_called_once()

    @patch.object(DatasetClient, "post")
    def test_get_organization_datasets(self, mock_post: MagicMock) -> None:
        """Test get organization datasets."""
        mock_post.return_value = {"data": {"datasets": [{"id": "1", "title": "Org Dataset"}]}}

        result = self.client.get_organization_datasets("org-123", limit=10)

        self.assertIsInstance(result, (list, dict))
        mock_post.assert_called_once()

    @patch.object(DatasetClient, "get")
    def test_search_with_filters(self, mock_get: MagicMock) -> None:
        """Test dataset search with filters."""
        mock_get.return_value = {"total": 5, "results": []}

        result = self.client.search(
            query="health",
            tags=["public-health"],
            sectors=["health"],
            status="PUBLISHED",
            access_type="OPEN",
        )

        self.assertEqual(result["total"], 5)
        mock_get.assert_called_once()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
Migration script to move geography data from metadata to the new geographies field.

This script:
1. Finds all datasets and usecases with geography metadata
2. Maps geography names to Geography model instances
3. Adds the geography relationships to the new ManyToMany field
4. Optionally removes the old metadata entries
5. Optionally updates the search index

Usage:
    python migrate_geography_metadata.py [--dry-run] [--remove-metadata] [--update-index] [--dataset-id ID] [--usecase-id ID] [--list-geographies]
    python migrate_geography_metadata.py [--dry-run] [--remove-metadata] [--update-index] [--datasets-only] [--usecases-only] [--list-geographies]

Options:
    --dry-run: Show what would be migrated without making changes
    --remove-metadata: Remove geography metadata after successful migration
    --update-index: Update search index after successful migration
    --dataset-id ID: Migrate only the dataset with the specified ID
    --usecase-id ID: Migrate only the usecase with the specified ID
    --datasets-only: Only migrate datasets (bulk migration)
    --usecases-only: Only migrate usecases (bulk migration)
    --list-geographies: List all available geographies in the database
"""

import argparse
import os
import subprocess
import sys
from typing import List, Set

import django

# Setup Django environment
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DataSpace.settings")
django.setup()

from api.models import (
    Collaborative,
    Dataset,
    DatasetMetadata,
    Geography,
    Metadata,
    UseCase,
    UseCaseMetadata,
)
from api.utils.enums import MetadataModels


def parse_geography_value(value: str) -> List[str]:
    """Parse geography value which might be comma-separated."""
    if not value:
        return []
    # Split by comma and strip whitespace
    return [geo.strip() for geo in value.split(",") if geo.strip()]


def find_geography_by_name(name: str) -> Geography | None:
    """Find geography by name (case-insensitive)."""
    try:
        return Geography.objects.filter(name__iexact=name).first()
    except Geography.DoesNotExist:
        return None


def migrate_dataset_geographies(
    dry_run: bool = False, remove_metadata: bool = False, dataset_id: str | None = None
) -> None:
    """Migrate geography metadata to geographies field for datasets."""
    print("\n" + "=" * 80)
    if dataset_id:
        print(f"MIGRATING DATASET GEOGRAPHIES FOR DATASET ID: {dataset_id}")
    else:
        print("MIGRATING DATASET GEOGRAPHIES")
    print("=" * 80)

    # Get geography metadata item
    try:
        geo_metadata_item = Metadata.objects.get(
            label="Geography", model=MetadataModels.DATASET
        )
    except Metadata.DoesNotExist:
        print("⚠️  No Geography metadata item found. Skipping dataset migration.")
        return

    # Find all datasets with geography metadata
    if dataset_id:
        dataset_metadata = DatasetMetadata.objects.filter(
            metadata_item=geo_metadata_item, dataset_id=dataset_id
        ).select_related("dataset", "metadata_item")
    else:
        dataset_metadata = DatasetMetadata.objects.filter(
            metadata_item=geo_metadata_item
        ).select_related("dataset", "metadata_item")

    total_datasets = dataset_metadata.count()
    if dataset_id:
        print(
            f"\nFound {total_datasets} geography metadata entries for dataset ID {dataset_id}"
        )
    else:
        print(f"\nFound {total_datasets} datasets with geography metadata")

    migrated_count = 0
    error_count = 0
    not_found_geographies: Set[str] = set()

    for ds_meta in dataset_metadata:
        dataset = ds_meta.dataset  # type: ignore
        geo_names = parse_geography_value(ds_meta.value)  # type: ignore

        if not geo_names:
            continue

        print(f"\n📦 Dataset: {dataset.title} (ID: {dataset.id})")
        print(f"   Geography metadata: {ds_meta.value}")  # type: ignore

        geographies_to_add = []
        for geo_name in geo_names:
            geography = find_geography_by_name(geo_name)
            if geography:
                geographies_to_add.append(geography)
                print(f"   ✓ Found: {geography.name} ({geography.type})")
            else:
                not_found_geographies.add(geo_name)
                print(f"   ✗ Not found: {geo_name}")

        if geographies_to_add:
            if not dry_run:
                # Add geographies to the dataset
                dataset.geographies.add(*geographies_to_add)
                print(f"   ✅ Added {len(geographies_to_add)} geographies to dataset")

                # Optionally remove metadata
                if remove_metadata:
                    ds_meta.delete()  # type: ignore
                    print(f"   🗑️  Removed geography metadata")

                migrated_count += 1
            else:
                print(f"   [DRY RUN] Would add {len(geographies_to_add)} geographies")
        else:
            error_count += 1
            print(f"   ⚠️  No valid geographies found for this dataset")

    print(f"\n{'=' * 80}")
    if dataset_id:
        print(f"Dataset Migration Summary for Dataset ID {dataset_id}:")
    else:
        print(f"Dataset Migration Summary:")
    print(f"  Total datasets processed: {total_datasets}")
    print(f"  Successfully migrated: {migrated_count}")
    print(f"  Errors/Skipped: {error_count}")
    if not_found_geographies:
        print(f"\n  Geographies not found in database:")
        for geo in sorted(not_found_geographies):
            print(f"    - {geo}")
    print(f"{'=' * 80}")


def migrate_usecase_geographies(
    dry_run: bool = False, remove_metadata: bool = False, usecase_id: int | None = None
) -> None:
    """Migrate geography metadata to geographies field for usecases."""
    print("\n" + "=" * 80)
    if usecase_id:
        print(f"MIGRATING USECASE GEOGRAPHIES FOR USECASE ID: {usecase_id}")
    else:
        print("MIGRATING USECASE GEOGRAPHIES")
    print("=" * 80)

    # Get geography metadata item
    try:
        geo_metadata_item = Metadata.objects.get(
            label="Geography", model=MetadataModels.USECASE
        )
    except Metadata.DoesNotExist:
        print("⚠️  No Geography metadata item found. Skipping usecase migration.")
        return

    # Find all usecases with geography metadata
    if usecase_id:
        usecase_metadata = UseCaseMetadata.objects.filter(
            metadata_item=geo_metadata_item, usecase_id=str(usecase_id)
        ).select_related("usecase", "metadata_item")
    else:
        usecase_metadata = UseCaseMetadata.objects.filter(
            metadata_item=geo_metadata_item
        ).select_related("usecase", "metadata_item")

    total_usecases = usecase_metadata.count()
    if usecase_id:
        print(
            f"\nFound {total_usecases} geography metadata entries for usecase ID {usecase_id}"
        )
    else:
        print(f"\nFound {total_usecases} usecases with geography metadata")

    migrated_count = 0
    error_count = 0
    not_found_geographies: Set[str] = set()

    for uc_meta in usecase_metadata:
        usecase = uc_meta.usecase  # type: ignore
        geo_names = parse_geography_value(uc_meta.value)  # type: ignore

        if not geo_names:
            continue

        print(f"\n📋 UseCase: {usecase.title} (ID: {usecase.id})")
        print(f"   Geography metadata: {uc_meta.value}")  # type: ignore

        geographies_to_add = []
        for geo_name in geo_names:
            geography = find_geography_by_name(geo_name)
            if geography:
                geographies_to_add.append(geography)
                print(f"   ✓ Found: {geography.name} ({geography.type})")
            else:
                not_found_geographies.add(geo_name)
                print(f"   ✗ Not found: {geo_name}")

        if geographies_to_add:
            if not dry_run:
                # Add geographies to the usecase
                usecase.geographies.add(*geographies_to_add)
                print(f"   ✅ Added {len(geographies_to_add)} geographies to usecase")

                # Optionally remove metadata
                if remove_metadata:
                    uc_meta.delete()  # type: ignore
                    print(f"   🗑️  Removed geography metadata")

                migrated_count += 1
            else:
                print(f"   [DRY RUN] Would add {len(geographies_to_add)} geographies")
        else:
            error_count += 1
            print(f"   ⚠️  No valid geographies found for this usecase")

    print(f"\n{'=' * 80}")
    if usecase_id:
        print(f"UseCase Migration Summary for UseCase ID {usecase_id}:")
    else:
        print(f"UseCase Migration Summary:")
    print(f"  Total usecases processed: {total_usecases}")
    print(f"  Successfully migrated: {migrated_count}")
    print(f"  Errors/Skipped: {error_count}")
    if not_found_geographies:
        print(f"\n  Geographies not found in database:")
        for geo in sorted(not_found_geographies):
            print(f"    - {geo}")
    print(f"{'=' * 80}")


def update_search_index(index_type: str | None = None) -> None:
    """Update the Elasticsearch search index."""
    print(f"\n🔄 Updating search index{' for ' + index_type if index_type else ''}...")

    try:
        # Run the Django management command to rebuild the search index
        result = subprocess.run(
            [sys.executable, "manage.py", "search_index", "--rebuild"],
            capture_output=True,
            text=True,
            cwd=os.getcwd(),
        )

        if result.returncode == 0:
            print("✅ Search index updated successfully!")
            if result.stdout.strip():
                print(f"Output: {result.stdout.strip()}")
        else:
            print("❌ Failed to update search index!")
            if result.stderr.strip():
                print(f"Error: {result.stderr.strip()}")

    except Exception as e:
        print(f"❌ Error updating search index: {str(e)}")


def list_available_geographies() -> None:
    """List all available geographies in the database."""
    print("\n" + "=" * 80)
    print("AVAILABLE GEOGRAPHIES IN DATABASE")
    print("=" * 80)

    geographies = Geography.objects.all().order_by("type", "name")

    if not geographies.exists():
        print("\n⚠️  No geographies found in database!")
        print("   Please populate geographies first using:")
        print("   python manage.py populate_geographies")
        return

    by_type: dict[str, list] = {}
    for geo in geographies:
        if geo.type not in by_type:  # type: ignore
            by_type[geo.type] = []  # type: ignore
        by_type[geo.type].append(geo)  # type: ignore

    for geo_type, geos in sorted(by_type.items()):
        print(f"\n{geo_type}:")
        for geo in geos:
            parent_info = f" (parent: {geo.parent_id.name})" if geo.parent_id else ""  # type: ignore
            print(f"  - {geo.name}{parent_info}")  # type: ignore

    print(f"\nTotal: {geographies.count()} geographies")
    print("=" * 80)


def main() -> None:
    parser = argparse.ArgumentParser(
        description="Migrate geography metadata to geographies field"
    )
    parser.add_argument(
        "--dry-run",
        action="store_true",
        help="Show what would be migrated without making changes",
    )
    parser.add_argument(
        "--remove-metadata",
        action="store_true",
        help="Remove geography metadata after successful migration",
    )
    parser.add_argument(
        "--list-geographies",
        action="store_true",
        help="List all available geographies in the database",
    )
    parser.add_argument(
        "--datasets-only",
        action="store_true",
        help="Only migrate datasets",
    )
    parser.add_argument(
        "--usecases-only",
        action="store_true",
        help="Only migrate usecases",
    )
    parser.add_argument(
        "--update-index",
        action="store_true",
        help="Update search index after successful migration",
    )
    parser.add_argument(
        "--dataset-id",
        type=str,
        help="Migrate only the dataset with the specified ID (UUID)",
    )
    parser.add_argument(
        "--usecase-id",
        type=int,
        help="Migrate only the usecase with the specified ID",
    )

    args = parser.parse_args()

    if args.list_geographies:
        list_available_geographies()
        return

    # Validate arguments
    if args.dataset_id and args.usecase_id:
        print("❌ Error: Cannot specify both --dataset-id and --usecase-id")
        sys.exit(1)

    if args.dataset_id and (args.datasets_only or args.usecases_only):
        print(
            "❌ Error: Cannot use --dataset-id with --datasets-only or --usecases-only"
        )
        sys.exit(1)

    if args.usecase_id and (args.datasets_only or args.usecases_only):
        print(
            "❌ Error: Cannot use --usecase-id with --datasets-only or --usecases-only"
        )
        sys.exit(1)

    print("\n" + "=" * 80)
    print("GEOGRAPHY METADATA MIGRATION SCRIPT")
    print("=" * 80)

    if args.dry_run:
        print("\n🔍 DRY RUN MODE - No changes will be made")

    if args.remove_metadata and not args.dry_run:
        print("\n⚠️  WARNING: Geography metadata will be DELETED after migration")
        response = input("Are you sure you want to continue? (yes/no): ")
        if response.lower() != "yes":
            print("Migration cancelled.")
            return

    # Run migrations
    if args.dataset_id:
        migrate_dataset_geographies(
            dry_run=args.dry_run,
            remove_metadata=args.remove_metadata,
            dataset_id=args.dataset_id,
        )
    elif args.usecase_id:
        migrate_usecase_geographies(
            dry_run=args.dry_run,
            remove_metadata=args.remove_metadata,
            usecase_id=args.usecase_id,
        )
    else:
        # Bulk migration mode
        if not args.usecases_only:
            migrate_dataset_geographies(
                dry_run=args.dry_run, remove_metadata=args.remove_metadata
            )

        if not args.datasets_only:
            migrate_usecase_geographies(
                dry_run=args.dry_run, remove_metadata=args.remove_metadata
            )

    print("\n" + "=" * 80)
    print("MIGRATION COMPLETE")
    print("=" * 80)

    if args.dry_run:
        print("\n💡 This was a dry run. Run without --dry-run to apply changes.")
    else:
        print("\n✅ Migration completed successfully!")

        # Update search index if requested and migration was successful
        if args.update_index:
            # Determine which index type to update based on what was migrated
            if args.dataset_id:
                update_search_index("datasets")
            elif args.usecase_id:
                update_search_index("usecases")
            else:
                # For bulk migrations, update based on what was actually migrated
                if not args.usecases_only:
                    update_search_index("datasets")
                if not args.datasets_only:
                    update_search_index("usecases")

        print("\n📝 Next steps:")
        print("   1. Verify the migrated data in the admin panel")
        if not args.update_index:
            print(
                "   2. Re-index Elasticsearch: python manage.py search_index --rebuild"
            )
        print("   3. Test the geography filters in the frontend")


if __name__ == "__main__":
    main()



import strawberry
import strawberry_django
from strawberry.types import Info

from api.models import Tag
from api.schema.base_mutation import (
    BaseMutation,
    DjangoValidationError,
    MutationResponse,
)
from api.utils.graphql_telemetry import trace_resolver
from authorization.permissions import IsAuthenticated


@strawberry.type
class Mutation:
    """Mutations for tags."""

    @strawberry_django.mutation(
        handle_django_errors=False, permission_classes=[IsAuthenticated]
    )
    @trace_resolver(
        name="delete_tag", attributes={"component": "tag", "operation": "mutation"}
    )
    def delete_tag(self, info: Info, tag_id: str) -> bool:
        """Delete a tag."""
        try:
            tag = Tag.objects.get(id=tag_id)
        except Tag.DoesNotExist:
            raise ValueError(f"Tag with ID {tag_id} does not exist.")
        tag.delete()
        return True

    @strawberry.mutation
    @BaseMutation.mutation(
        permission_classes=[IsAuthenticated],
        trace_name="delete_tags",
        trace_attributes={"component": "tag"},
    )
    def delete_tags(self, info: Info, tag_ids: list[str]) -> MutationResponse[bool]:
        """Delete multiple tags."""

        tags = Tag.objects.filter(id__in=tag_ids)
        if not tags.exists():
            raise DjangoValidationError(f"Tags with IDs {tag_ids} do not exist.")
        tags.delete()
        return MutationResponse.success_response(True)



import logging

import structlog

# Configure structlog
structlog.configure(
    processors=[
        structlog.stdlib.filter_by_level,
        structlog.stdlib.add_logger_name,
        structlog.stdlib.add_log_level,
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.TimeStamper(fmt="iso"),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
        structlog.processors.UnicodeDecoder(),
        structlog.processors.JSONRenderer(),
    ],
    context_class=dict,
    logger_factory=structlog.stdlib.LoggerFactory(),
    wrapper_class=structlog.stdlib.BoundLogger,
    cache_logger_on_first_use=True,
)

# Create and configure the logger
logger = structlog.get_logger("dataspace")

# Set the logging level
logger.setLevel(logging.INFO)

# Add console handler if needed
console_handler = logging.StreamHandler()
console_handler.setLevel(logging.INFO)
logger.addHandler(console_handler)



# """DataSpace URL Configuration

# The `urlpatterns` list routes URLs to views. For more information please see:
#     https://docs.djangoproject.com/en/4.0/topics/http/urls/
# Examples:
# Function views
#     1. Add an import:  from my_app import views
#     2. Add a URL to urlpatterns:  path('', views.home, name='home')
# Class-based views
#     1. Add an import:  from other_app.views import Home
#     2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
# Including another URLconf
#     1. Import the include() function: from django.urls import include, path
#     2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
# """
# from django.conf.urls.static import static
# from django.conf.urls import include
# from django.urls import path
# # from django.template.defaulttags import url
# from rest_framework.routers import DefaultRouter

# from DataSpace import settings
# from search.views import DatasetDocumentView, DatasetCompoundSearchBackendDocumentViewSet

# router = DefaultRouter()
# router.register(r'datasets',
#                     DatasetDocumentView,
#                     basename='datasetdocument')
# router.register(r'sdatasets',
#                     DatasetCompoundSearchBackendDocumentViewSet,
#                     basename='sdatasetdocument')

# urlpatterns = [
#                   path('', include(router.urls)),
#               ] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
